from MAST.structopt.generate.surface import get_surface_indiv
from MAST.structopt.generate.defect import get_defect_indiv
from MAST.structopt.generate.crystal import get_crystal_indiv
from MAST.structopt.generate import gen_pop_box, gen_pop_sphere
from MAST.structopt.generate.Individual import Individual

def get_population(Optimizer):
    """
	Function to generate a population of structures.
	Inputs:
		Optimizer = structopt Optimizer class object
	Outputs:
		pop = List of structopt Individual class objects containing new structures.
	"""
    index1 = 0
    pop = []
    for i in range(Optimizer.nindiv):
        if Optimizer.structure == 'Defect':
            individ = get_defect_indiv(Optimizer)
        elif Optimizer.structure == 'Surface':
            individ = get_surface_indiv(Optimizer)
        elif Optimizer.structure == 'Crystal':
            individ = get_crystal_indiv(Optimizer)
        else:
            if 'sphere' in Optimizer.generate_flag:
                ind = gen_pop_sphere(Optimizer.atomlist,Optimizer.size)
            else:
                ind = gen_pop_box(Optimizer.atomlist,Optimizer.size)
            individ = Individual(ind)
        individ.index = index1
        if Optimizer.genealogy: 
            individ.history_index = repr(index1)
        Optimizer.output.write('Generated cluster individual with natoms = '+
            repr(individ[0].get_number_of_atoms())+'\n')
        pop.append(individ)
        index1=index1+1
    # Generate new atomlist concentrations based on cluster+box
    if Optimizer.structure == 'Defect':
        if Optimizer.alloy:
            concents=[]
            for ind in pop:
                cs=[]
                for sym,c,u,m in Optimizer.atomlist:
                    sylen=[atm for atm in ind[0] if atm.symbol==sym]
                    cs.append(len(sylen))
                concents.append(cs)
            natmlist=[0]*len(Optimizer.atomlist)
            for i in range(len(concents[0])):
                alls=[cs[i] for cs in concents]
                avgall=int(sum(alls)/len(alls))
                if avgall>=Optimizer.atomlist[i][1]:
                    natmlist[i]=(Optimizer.atomlist[i][0], avgall,
                        Optimizer.atomlist[i][2],Optimizer.atomlist[i][3])
                else:
                    natmlist[i]=(Optimizer.atomlist[i][0], Optimizer.atomlist[i][1],
                        Optimizer.atomlist[i][2],Optimizer.atomlist[i][3])
        else:
            natmlist=[0]*len(Optimizer.atomlist)
            for i in range(len(Optimizer.atomlist)):
                atms1=[inds for inds in pop[0][0] if inds.symbol==Optimizer.atomlist[i][0]]
                natmlist[i]=(Optimizer.atomlist[i][0], len(atms1),Optimizer.atomlist[i][2],
                    Optimizer.atomlist[i][3])
        Optimizer.atomlist=natmlist
        Optimizer.output.write('\n\nNew atomlist concentrations based on cluster+box = '+
            repr(Optimizer.atomlist)+'\n')
    return pop



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
# Purpose: Submit a job to the queue. This function is highly platform-specific and may need to be modified.
# 10/25/12 TTM created
# 12/6/12 TTM added UKY DLX commands
import os
import sys
import math #TA to calc num_nodes
import time
import importlib
import subprocess
import shutil
import logging
from MAST.utility.mastfile import MASTFile
from MAST.utility import MASTError
from MAST.utility import dirutil
from MAST.utility import loggerutils

mast_control = os.getenv("MAST_CONTROL")
platform_file = MASTFile("%s/set_platform" % mast_control)
mast_platform = platform_file.data[0].strip()

myqc = "MAST.submit.platforms.%s.queue_commands" % mast_platform
my_queue_commands = importlib.import_module(myqc)

def direct_shell_command(scriptname="submit.sh"):
    myds = my_queue_commands.direct_shell_command()
    return "%s %s" % (myds, scriptname)

def queue_submission_command(scriptname="submit.sh"):
    myqs = my_queue_commands.queue_submission_command()
    return "%s %s" % (myqs, scriptname)

def write_to_submit_list(mydir):
    """Write an entry to the submission list in 
        $MAST_CONTROL/submitlist
        Args:
            mydir <str>: Directory which includes submission
                        script submit.sh for a single 
                        calculation to be submitted to the
                        queue
    """
    control=dirutil.get_mast_control_path()
    submitlist=os.path.join(control, "submitlist")
    if os.path.isfile(submitlist):
        submitfile=MASTFile(submitlist)
    else:
        submitfile=MASTFile()
    submitfile.data.append(mydir + "\n")
    submitfile.to_file(submitlist)
    return

def submit_from_submission_list():
    """Submit all entries from the submission list at
        $MAST_CONTROL/submitlist
        Adds a job number to the top of the "jobids" file in each
        ingredient directory.
    """
    control=dirutil.get_mast_control_path()
    submitlist=os.path.join(control, "submitlist")
    if not os.path.isfile(submitlist):
        print "No submission list at %s" % submitlist
        return
    submitfile=MASTFile(submitlist)
    subentries=list(submitfile.data)
    subentries.sort()
    submitted=dict()
    subcommand = queue_submission_command()
    for subentry in subentries: # each entry is a directory name
        if len(subentry) == 0:
            continue
        subentry = subentry.strip()
        if len(subentry) == 0:
            continue
        if not os.path.isdir(subentry):
            continue
        elif subentry in submitted.keys():
            continue
        lastjobid = get_last_jobid(subentry)
        jstatus="not yet determined"
        if not (lastjobid == None):
            jstatus = get_job_status_from_queue_snapshot(subentry, lastjobid)
            if jstatus.lower() in ['r','q','h','e']: #running, queued, held, error
                submitted[subentry]="Already on queue with status %s" % jstatus
                continue
        os.chdir(subentry)
        subme=subprocess.Popen(subcommand, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        subme.wait()
        status=subme.communicate()[0]
        write_to_jobids_file(subentry, status)
        submitted[subentry]=status
    print_submitted_dict(submitted)
    os.chdir(control)

def write_to_jobids_file(subentry, status):
    """Write the job id to a jobids file in the ingredient directory.
        Args:
            subentry <str>: Folder from which the job was submitted 
                            (ingredient folder)
            status <str>: Job submission response from queueing system
    """
    jobid = extract_submitted_jobid(status)
    jpath = "%s/jobids" % subentry
    if not os.path.isfile(jpath):
        jfile = MASTFile()
    else:
        jfile = MASTFile(jpath)
    jfile.data.insert(0,str(jobid) + '\n')
    jfile.to_file(jpath)
    return

def get_job_status_from_queue_snapshot(ingpath, jobid):
    """Match a jobid to the queue_snapshot
        Args:
            ingpath <str>: ingredient path
            jobid <int>: job id
        Returns:
            Queue status for the job, according to $MAST_CONTROL/queue_snapshot
    """
    myqs = MASTFile("%s/queue_snapshot" % os.getenv("MAST_CONTROL"))
    jobstatus = ""
    for qsline in myqs.data:
        if str(jobid) in qsline:
            jobstatus = queue_status_from_text(jobid, qsline)
    return jobstatus

def get_last_jobid(ingpath):
    """Get the last job id from the jobids file in the ingredient path
        Args:
            ingpath <str>: ingredient path
        Returns:
            jobid <int>: job id, if available
            None if no jobids file was found
    """
    jpath = "%s/jobids" % ingpath
    if not os.path.isfile(jpath):
        return None
    jfile = MASTFile(jpath)
    if len(jfile.data) == 0:
        return None
    if 'None' in jfile.data[0]:
        return None
    return int(jfile.data[0].strip())

def clear_submission_list():
    """Clear all entries from the submission list at
        $MAST_CONTROL/submitlist
    """
    control=dirutil.get_mast_control_path()
    submitlist=os.path.join(control, "submitlist")
    if not os.path.isfile(submitlist):
        print "No submission list at %s" % submitlist
        return
    submitfile=MASTFile(submitlist)
    submitfile.data=list()
    submitfile.data.append("\n")
    submitfile.to_file(submitlist)

def print_submitted_dict(submitted):
    """Print a dictionary of all runs submitted into
        $MAST_CONTROL/submitted
        Args:
            submitted <dict>: Dictionary of submitted runs,
                with key as the directory name.
    """
    control=dirutil.get_mast_control_path()
    subprint=os.path.join(control, "submitted")
    subrecent=os.path.join(control, "just_submitted")
    if os.path.isfile(subprint):
        subprintfile=MASTFile(subprint)
    else:
        subprintfile=MASTFile()
    subrecentfile=MASTFile()
    keylist=submitted.keys()
    keylist.sort()
    subprintfile.data.append(time.asctime()+ "\n")
    subrecentfile.data.append(time.asctime()+ "\n")
    for key in keylist:
        subprintfile.data.append("%s:%s\n" % (key, submitted[key]))
        subrecentfile.data.append("%s:%s\n" % (key, submitted[key]))
    subprintfile.to_file(subprint)
    subrecentfile.to_file(subrecent)
    return
    


def queue_status_from_text(jobid, queuetext):
    """
        Returns the queue status from a line with the job ID information
        INPUTS:
            jobid <int> = job ID
            qstr <str> = queue snapshot line containing job ID
        OUTPUTS:
            <str> = queue status 
            'E': Error
            'X': Not found
            'R': Running
            'W': Waiting
    """
    return my_queue_commands.queue_status_from_text(jobid, queuetext)

def extract_submitted_jobid(string):
    """
        Extract the job ID from a string returned when the job is submitted
        INPUTS:
            string <str> = string with job ID somewhere in it
        OUTPUTS:
            <int> = job ID as integer
    """
    return my_queue_commands.extract_submitted_jobid(string)

def queue_snap_command():
    """
        Return the command for producing a queue snapshot.
        INPUTS:
            None
        OUTPUTS:
            Command for producing a queue snapshot
    """
    return my_queue_commands.queue_snap_command()

def get_job_error_file(ingpath):
    """
        Return the job error file full path
            Args:
                ingpath <str>: ingredient path
    """
    logger = logging.getLogger('mast')
    logger = loggerutils.add_handler_for_control(logger)
    jobid = get_last_jobid(ingpath)
    tryfile = my_queue_commands.get_approx_job_error_file(jobid)
    #logger.info("Try this search string: %s" % tryfile)
    if not os.path.isdir(ingpath):
        return None
    dircontents = os.listdir(ingpath)
    tryitems = list()
    for diritem in dircontents:
        if tryfile in diritem:
            tryitems.append(os.path.join(ingpath, diritem))
    if len(tryitems) == 0:
        logger.warning("No job error file found for %s" % ingpath)
        return None
    if len(tryitems) > 1:
        logger.warning("More than one job error file found for %s. Using first file %s" % (ingpath, tryitems[0]))
    return tryitems[0]
    




from ase import Atom, Atoms

def read_xyz(fileobj,n=-1,data=False):
    """
    Function to read multi-atom xyz file with data
    Inputs:
        fileobj = String containing file name or file object
        n = Integer indicating number for structure from xyz file
            to read. Default is last structure in file.
            Or String=='All' which will output all structures in file
        data = True/False boolean indicating whether or not to output 
            any data strings read from xyz file
    Outputs:
        atmslist = ASE Atoms class object containing structure from file
            or list of Atoms objects if n=='All'
        datalist(Optional) = String of data read from xyz file or list 
            of data strings read from xyz file
    """
    if isinstance(fileobj, str):
        fileobj = open(fileobj,'r')
    lines = fileobj.readlines()
    atmslist = []
    datalist = []
    while len(lines)>0:
        natoms = int(lines[0])
        datalist.append(lines[1])
        atm1 = Atoms()
        i =- 1
        for i in range(natoms):
            a = lines[i+2].split()
            sym = a[0]
            position = [float(a[1]),float(a[2]),float(a[3])]
            atm1.append(Atom(symbol=sym,position=position))
        atmslist.append(atm1)
        lines = lines[i+3::]
    if n == 'All':
        if data == True:
            return atmslist, datalist
        else:
            return atmslist
    else:
        if data == True:
            return atmslist[n],datalist[n]
        else:
            return atmslist[n]




import random
import numpy

def atoms_remove(indiv, Optimizer):
    """Move function to alter an individual by removing an atom or group of atoms.
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    nato=indiv[0].get_number_of_atoms()-2.0
    for i in range(random.randint(1,nato)):
        indiv[0].pop(random.choice(indiv[0]).index)
    Optimizer.output.write('Remove atom mutation\n')
    if Optimizer.structure=='Crystal':
        cell=indiv[0].get_cell()
        a, b, c = cell
        an, bn, cn = [numpy.linalg.norm(v) for v in cell]
        cell[0]=a/an
        cell[1]=b/bn
        cell[2]=c/cn
        indiv[0].set_cell(cell*[(indiv[0].get_number_of_atoms())**0.33*Optimizer.r_ab])
    muttype='RA'+repr(nato-indiv[0].get_number_of_atoms()+2)
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    Optimizer.output.write(repr(indiv[0].get_positions())+'\n')
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


'''Predator functions for use in optimizer'''
# from adapting import *
# from energy_cluster import *
# from fingerprint_niche import *
# from fitpred_bests import *
# from fitpred_new import *
# from fitpred import *
# from mutation_dups_energy import *
# from mutation_dups import *
# from mutation_dups_zp import *
# from mutation_dups_quench import *


import logging


def initialize_logger(filename="default.log"):
    logger = logging.getLogger('')
    handler = logging.FileHandler(filename)
    formatter = logging.Formatter('%(asctime)s : <%(pathname)s:%(lineno)d> : %(levelname)s : %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)
    return logger


import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import numpy

def view(cluster, colorlist=[], nints=0):
	fig = plt.figure()
	ax = fig.add_subplot(111, projection='3d')
	syms = list(set(cluster.get_chemical_symbols()))
	size=1
	for sym in syms:
		if sym in [s for s,c in colorlist]:
			colors = colorlist[[i for i,c in enumerate(colorlist) if c[0]==sym][0]][1]
		else:
			colors = numpy.random.rand(3,1)
			colorlist.append([sym,colors])
		xs = []
		ys = []
		zs = []
		for atm in cluster:
			if atm.index >= nints:
				if atm.symbol == sym:
					xs.append(atm.position[0])
					ys.append(atm.position[1])
					zs.append(atm.position[2])
					size = atm.mass**2
		ax.scatter(xs,ys,zs,s=size,c=colors)
	if 'defect' in [s for s,c in colorlist]:
		colors = colorlist[[i for i,c in enumerate(colorlist) if c[0]=='defect'][0]][1]
	#else:
	#	colors = numpy.random.rand(3,1)
	#	colorlist.append(['defect',colors])
	for sym in syms:
		xs = []
		ys = []
		zs = []
		for atm in cluster:
			if atm.index < nints:
				if atm.symbol == sym:
					xs.append(atm.position[0])
					ys.append(atm.position[1])
					zs.append(atm.position[2])
					size = atm.mass**2
		ax.scatter(xs,ys,zs,s=size,c=colors)
	ax.set_xlabel('X Position')
	ax.set_ylabel('Y Position')
	ax.set_zlabel('Z Position')
	plt.show()
	return colorlist


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-05-12 by Zhewen Song
##############################################################
import os, re
import logging
import shlex
from MAST.utility import InputOptions
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import Metadata
from MAST.utility import loggerutils
from pymatgen.core.structure import Structure
from MAST.recipe import recipeutility as ru
from MAST.recipe.recipeplan import RecipePlan
from MAST.ingredients.baseingredient import BaseIngredient
ALLOWED_KEYS = {
                  'recipeFile'     : (str, None, 'Personalized recipe file'),\
                  'inputOptions'   : (InputOptions, None, 'Input options parsed using input parser'),\
                  'structure'      : (Structure, None, 'Structure to be used to create the ingredient objects'),\
                  'ingredientsDict': (dict, dict(), 'Dictionary of ingredients'),\
                  'workingDirectory': (str, None, 'Working directory')
               }

DATA_PATH = "~/test_dir/"

class RecipeSetup(MASTObj):
    """Parses the personalized recipe file,
        creates the recipe plan object,
        and prepares the ingredient directories.
        Attributes:
            self.recipe_file <str>: Recipe file name
            self.input_options <InputOptions>: Input options for recipe
            self.structure <pymatgen Structure>: Initial structure in recipe
            self.work_dir <str>: Working (recipe) directory)
            self.logger <logging Logger>: MAST monitor and MAST setup-level log
            self.recipe_logger <logging Logger>: Recipe-level log
    """
    def __init__(self, **kwargs):
        MASTObj.__init__(self, ALLOWED_KEYS, **kwargs)
        self.recipe_file    = self.keywords['recipeFile']
        self.input_options  = self.keywords['inputOptions']
        self.structure      = self.keywords['structure']
        self.work_dir    = self.keywords['workingDirectory']
        self.logger = logging.getLogger('mastmon')
        self.logger = loggerutils.add_handler_for_control(self.logger)
        self.recipe_logger = logging.getLogger(self.work_dir)
        self.recipe_logger = loggerutils.add_handler_for_recipe(self.work_dir, self.recipe_logger)

        self.metafile = Metadata(metafile='%s/metadata.txt' % self.work_dir)
        self.recipe_logger.info('Setting up the recipe based on %s' % (self.recipe_file))

    def get_my_ingredient_options(self, name, ingredient_type):
        """Creates the ingredient based on the ingredient type.
            Notes:

            GRJ 6/19/2013: If the ingredient has not be specified in the input
                           file we create it here, and append on the defaults.
                           In addition, the defaults are appended on here, rather
                           than in InputParser (since each ingredient is made here
                           this makes more sense).
            TTM 8/19/13: Unspecified ingredient is "default"
        """
        self.recipe_logger.info('Initializing ingredient %s of type %s' % (name, ingredient_type))
        global_defaults = self.input_options.get_item('ingredients', 'global')

        if (ingredient_type not in self.input_options.get_section_keys('ingredients')):
            self.recipe_logger.info('Ingredient type %s has not be specified in the input file.' % ingredient_type)
            self.recipe_logger.info('Using defaults from ingredients_global.')
            self.input_options.set_item('ingredients', ingredient_type, global_defaults)
        else:
            self.recipe_logger.info('Copying over defaults from ingredients_global for ingredient %s.' % ingredient_type)
            ing_opt = self.input_options.get_item('ingredients', ingredient_type)
            for glob_key, glob_value in global_defaults.items():
                if glob_key not in ing_opt:
                    ing_opt[glob_key] = glob_value
            self.input_options.set_item('ingredients', ingredient_type, ing_opt)

        #self.program = self.input_options.options['ingredients'][ingredient_type]['mast_program']

        ingredient_name = os.path.join(self.work_dir, name)
        pkey_d = self.input_options.get_item('ingredients', ingredient_type).copy()
        mydata = self.metafile.read_data(os.path.basename(ingredient_name)).split(',')
        defect_label=""
        neb_label=""
        charge=""
        phonon_label=""

        if 'defect_' in name.lower():
            for datum in mydata:
                if 'defect_label' in datum:
                    defect_label = datum.split(':')[-1].strip()
                    if not 'defects' in self.input_options.options.keys():
                        raise MASTError(self.__class__.__name__, "No defects section in input file. Error setting up recipe %s." % self.work_dir)
                    defdict = self.input_options.get_item('defects','defects')
                    if not defect_label in defdict.keys():
                        raise MASTError(self.__class__.__name__, "No such label %s found in the defects section dictionary." % defect_label)
                    mydefdict = dict()
                    mydefdict['mast_defect_settings'] = defdict[defect_label]
                    pkey_d.update(mydefdict)
                    break
            if defect_label == "":
                raise MASTError(self.__class__.__name__, "No defect label for %s found in ingredient's metadata.txt" % ingredient_name)
                    
        if 'inducedefect' not in ingredient_type:
            if 'q=' in name.lower():
                for datum in mydata:
                    if 'charge' in datum:
                        charge = int(datum.split(':')[-1])
                        pkey_d['mast_charge'] = charge
                        break

        if 'neb_' in name.lower():
            for datum in mydata:
                if 'neb_label' in datum:
                    neb_label = datum.split(':')[-1].strip()
                    if not 'neb' in self.input_options.options.keys():
                        raise MASTError(self.__class__.__name__, "No neb section in input file. Error setting up recipe %s." % self.work_dir)
                    nebdict = self.input_options.get_item('neb','nebs')
                    if not neb_label in nebdict.keys():
                        raise MASTError(self.__class__.__name__, "No such label %s found in the neb section dictionary." % neb_label)
                    mynebdict = dict()
                    mynebdict['mast_neb_settings'] = nebdict[neb_label]
                    pkey_d.update(mynebdict)
                    break
            if neb_label == "":
                raise MASTError(self.__class__.__name__, "No neb label for %s found in ingredient's metadata.txt" % ingredient_name)

            if 'neb' in name.lower():
                pkey_d.update(self.input_options.options['neb'])
        #if 'phonon' in self.input_options.options.keys():
        #    pkey_d.update(self.input_options.options['phonon'])
        if 'phonon_' in name.lower():
            for datum in mydata:
                if 'phonon_label' in datum:
                    phonon_label = datum.split(':')[-1].strip()
                    def_or_neb_label = phonon_label.split('_')[0]
                    phonon_key = phonon_label.split('_')[-1]
                    defdict = self.input_options.get_item('defects','defects')
                    nebdict = self.input_options.get_item('neb','nebs')
                    phdict=dict()
                    if def_or_neb_label in defdict.keys():
                        phdict['mast_phonon_settings'] = defdict[def_or_neb_label]['phonon'][phonon_key]
                    elif def_or_neb_label in nebdict.keys():
                        phdict['mast_phonon_settings'] = nebdict[def_or_neb_label]['phonon'][phonon_key]
                    else:
                        raise MASTError(self.__class__.__name__, "Neither defect nor neb dictionaries have phonon key %s for ingredient %s." % (phonon_key, name))
                    pkey_d.update(phdict)
                    break
            if phonon_label == "":
                raise MASTError(self.__class__.__name__, "No phonon label for %s found in ingredient's metadata.txt" % ingredient_name)
        if 'chemical_potentials' in self.input_options.options.keys():
            chempotdict=dict()
            chempotdict['mast_chemical_potentials']=self.input_options.options['chemical_potentials']
            pkey_d.update(chempotdict)

        allopt=dict()
        allopt['name'] = ingredient_name
        #allopt['program'] = self.program
        allopt['structure'] = self.structure
        allopt['program_keys'] = pkey_d
        self.recipe_logger.info(pkey_d)
        return allopt
    def get_method_from_ingredient_type(self, ingredtype, methodtype=""):
        """Get write, ready, run, complete, and update methods from
            an ingredient type like volrelax_to_singlerun
            Args:
                ingredtype <str>: ingredient type
                methodtype <str>: method type ("mast_write_method", etc.)
            Returns:
                methodname <str>: method name ("write_default", etc.)
        """
        raise NotImplementedError
        mdict=self.input_options.get_item("ingredients", ingredtype)
        if methodtype in mdict.keys():
            return mdict[methodtype]
        else:
            mdict=self.input_options.get_item("ingredients","global")
            if methodtype in mdict.keys():
                return mdict[methodtype]
            else:
                return None


    def create_recipe_plan(self):
        """Creates a recipe object which has the ingredients and dependency information
        """
        import inspect
        #'GRJ DEBUG: %s.%s' % (self.__class__.__name__, inspect.stack()[0][3])
        [how_to_update, parents_to_check, how_to_run, recipe_name] = ru.read_recipe(self.recipe_file)

        recipe_obj = RecipePlan(recipe_name, self.work_dir)
        ingredlist = how_to_run.keys()
        for ingred in ingredlist:
            self.update_top_meta_for_ingred(ingred)
            ingredtype = how_to_run[ingred]
            recipe_obj.ingredients[ingred]="I" #set all to initialized
            recipe_obj.ingred_input_options[ingred] = self.get_my_ingredient_options(ingred, ingredtype)
            if not os.path.isdir(os.path.join(self.work_dir, ingred)):
                self.create_ingredient(recipe_obj.ingred_input_options[ingred])
            recipe_obj.write_methods[ingred] = self.get_method_list(ingredtype, "mast_write_method")
            recipe_obj.ready_methods[ingred] = self.get_method_list(ingredtype, "mast_ready_method")
            recipe_obj.run_methods[ingred] = self.get_method_list(ingredtype, "mast_run_method")
            recipe_obj.complete_methods[ingred] = self.get_method_list(ingredtype, "mast_complete_method")
            recipe_obj.update_methods[ingred] = dict()
            for ichild in how_to_update[ingred].keys():
                updingredtype = how_to_update[ingred][ichild]
                recipe_obj.update_methods[ingred][ichild] = self.get_method_list(updingredtype, "mast_update_children_method")
            recipe_obj.parents_to_check = dict(parents_to_check)
        recipe_obj.summary_options = self.get_summary_options()
        return recipe_obj

    def update_top_meta_for_ingred(self, myingred):
        """Update the top metafile for an ingredient.
            Args:
                myingred <str>: ingredient name
        """
        datalist=list()
        datalist.append("ingredient type: %s " % myingred)
        scaling = re.search(r'_\d*x\d*x\d*',myingred)
        if scaling: 
            scalingsize=scaling.group().split('_')[1]
            datalist.append("scaling_size: %s" % scalingsize)
            d_scaling = self.input_options.get_item("structure","scaling")
            kpoints = d_scaling[scalingsize]
            datalist.append("kpoints: %s %s %s" % (kpoints[0],kpoints[1],kpoints[2]) )
        if 'defect_' in myingred:
            if scaling:
                defectlabel = myingred.split('defect_')[1].split('_')[1]
            else: 
                defectlabel = myingred.split('defect_')[1].split('_')[0]
            if defectlabel.isdigit():
                defectlabel = "defect_" + defectlabel
            datalist.append("defect_label: %s" % defectlabel)
        if 'q=' in myingred:
            chargestr = myingred.split('q=')[1].split('_')[0]
            if chargestr[0] == 'p':
                chargelabel=chargestr[1:]
            elif chargestr[0] == 'n':
                chargelabel='-'+chargestr[1:]
            else:
                chargelabel=chargestr
            datalist.append("charge: %s" % chargelabel)
        if 'neb_' in myingred:
            if scaling:
                neblabel = myingred.split('neb_')[1].split('_')[1]
            else:
                neblabel = myingred.split('neb_')[1].split('_')[0]
            datalist.append("neb_label: %s" % neblabel)
        if 'phonon_' in myingred:
            labels = myingred.split('phonon_')[1].split('_')
            try: labels.remove('parse')
            except ValueError: pass
            if scaling: labels.remove(scalingsize)
            phononlabel = '_'.join(labels)
            datalist.append("phonon_label: %s" % phononlabel)
        data=','.join(datalist)
        self.metafile.write_data(myingred, data)

    def create_ingredient(self, my_ingred_input_options):
        """Create the ingredient directory and metadata file.
            Args:
                my_ingred_input_options <Input Options>: single ingredient
                                                            input options
        """
        allowed_keys = {
            'name' : (str, str(), 'Name of optimization directory'),
            'program': (str, str(), 'DFT program, e.g. "vasp"'),
            'program_keys': (dict, dict(), 'Dictionary of program keywords'),
            'structure': (Structure, None, 'Pymatgen Structure object')
            }


        myingred = BaseIngredient(allowed_keys, 
                        name=my_ingred_input_options['name'],
                        structure=my_ingred_input_options['structure'],
                        program_keys=my_ingred_input_options['program_keys'])
        myingred.write_directory()
        return


    def start(self):
        """Starts the setup process, parse the recipe file
           Use the input options and recipe info to
           create directories and classes required
        """
        if self.recipe_file is None:
            raise MASTError(self.__class__.__name__, "Recipe file not provided!")

        if not os.path.exists(self.recipe_file):
            raise MASTError(self.__class__.__name__, "Recipe file not Found!")

        if not self.input_options:
            raise MASTError(self.__class__.__name__, "Input Options not provided!")


        #print 'DEBUG:, ingredients info =',
        #for ingredient, value in ingredients_info.items():
        #    print ingredient, value
        recipe_plan = self.create_recipe_plan()
        return recipe_plan

    def get_method_list(self, ingredtype, methodtype):
        """Get a method list. Methods should be of
            the form:
                methodname arguments
            where methodname is the method name, and arguments
            are the arguments, as though listed through
            a command line.
            For example, copy_file CONTCAR POSCAR
            Multiple methods may exist with the same keyword
            in the ingredients section, and all will be applied.
            Methods should be separated by a semicolon:
            mast_update_children_method copy_file CONTCAR POSCAR; softlink_file WAVECAR
            Args:
                ingredtype <str>: ingredient type
                methodtype <str>: method type ("mast_write_method", etc.)
            Returns:
                mlist <list>: nested list containing entries
            [[method1name, arg1, arg2,...],[method2name...],...]
        """
        ioptdict=self.input_options.get_item("ingredients", ingredtype)
        if methodtype in ioptdict.keys():
            unparsed = ioptdict[methodtype]
        else:
            globaldict=self.input_options.get_item("ingredients","global")
            if methodtype in globaldict.keys():
                unparsed = globaldict[methodtype]
            else:
                raise MASTError(self.__class__.__name__,"No method type %s in either ingredient type %s or global ingredient." % (methodtype, ingredtype))
        splitmethods = unparsed.split(";")
        mlist = list()
        #self.logger.info(splitmethods)
        for method_arg_item in splitmethods:
            method_arg_item = method_arg_item.strip()
            method_arg_list = shlex.split(method_arg_item)
            methodname = method_arg_list[0].strip()
            arglist = list()
            for argitem in method_arg_list[1:]:
                arglist.append(argitem.strip())
            minputs = list()
            minputs.append(methodname)
            minputs.extend(arglist)
            mlist.append(minputs)
            #if not methodname in mdict.keys():
            #    mdict[methodname]=list(arglist)
            #else:
            #    self.logger.warning("Duplicate entry in method %s for ingredient type %s; ignoring the duplicate." % (methodtype, ingredtype))
        #self.logger.info(mlist)
        return mlist
        #return mdict

    def get_summary_options(self):
        """Get the summary options and give them to the recipe plan."""
        sum_opts = dict()
        if 'summary' in self.input_options.options.keys():
            sumkeys = self.input_options.get_section_keys("summary")
            for sumkey in sumkeys:
                sum_opts[sumkey] = self.input_options.get_item("summary",sumkey)
        return sum_opts



"""Tests for Defectformationenergy"""

from MAST.utility.defect_formation_energy.defectformationenergy import DefectFormationEnergy

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
import shutil
from MAST.utility import MASTFile
testname="defect_formation_energy_test"
testdir = dirutil.get_test_dir(testname)
oldarchive = os.getenv("MAST_ARCHIVE")
      
class TestDefectformationenergy(unittest.TestCase):

    def setUp(self):
        os.environ['MAST_ARCHIVE'] = os.path.join(testdir,'archive')
        os.chdir(testdir)

    def tearDown(self):
        os.environ['MAST_ARCHIVE'] = oldarchive
        if os.path.isdir("GaAs_defects_AsGa_recipe_defects_20131125T220427_dfe_results"):
            shutil.rmtree("GaAs_defects_AsGa_recipe_defects_20131125T220427_dfe_results")

    def test_dfe_tool(self):
        import subprocess
        mydfetest=subprocess.Popen(["mast_defect_formation_energy"],shell=True)
        mydfetest.wait()
        compare_walk = dirutil.walkfiles("compare_results")
        res_walk = dirutil.walkfiles("GaAs_defects_AsGa_recipe_defects_20131125T220427_dfe_results")
        compare_walk.sort()
        res_walk.sort()
        self.assertEqual(len(compare_walk), len(res_walk))
        for idx in range(0,len(compare_walk)):
            compfile = MASTFile(compare_walk[idx])
            myfile = MASTFile(res_walk[idx])
            self.assertEqual(compfile.data, myfile.data)

    def test___init__(self):
        raise SkipTest
        #self.testclass.__init__(directory=None, plot_threshold=0.01)

    def test__calculate_defect_formation_energies(self):
        raise SkipTest
        recipepath = os.path.join(testdir, 'archive','GaAs_defects_AsGa_recipe_defects_20131125T220427')
        mydfe = DefectFormationEnergy(directory=recipepath)
        mydfe._calculate_defect_formation_energies() 
        self.assertEqual(True,True)
        #self.testclass._calculate_defect_formation_energies()

    def test_get_total_energy(self):
        raise SkipTest
        #self.testclass.get_total_energy(directory)

    def test_get_fermi_energy(self):
        raise SkipTest
        #self.testclass.get_fermi_energy(directory)

    def test_get_structure(self):
        raise SkipTest
        #self.testclass.get_structure(directory)

    def test_get_potential_alignment(self):
        raise SkipTest
        #self.testclass.get_potential_alignment(perf_dir, def_dir)

    def test_get_defect_formation_energies(self):
        raise SkipTest
        #self.testclass.get_defect_formation_energies()

    def test_defect_formation_energies(self):
        raise SkipTest
        #self.testclass.defect_formation_energies()

    def test_dfe(self):
        raise SkipTest
        #self.testclass.dfe()

    def test_print_table(self):
        raise SkipTest
        #self.testclass.print_table()



'''Functions for crossover in optimizer'''
# from CCP_Oct_Tet import *
# from clustbx import *
# from cxtp import *
# from cxtpa import *
# from cxtpc import *
# from newclus import *
# from randalloybox import *
# from rotct_rand_clus import *
# from rotct_rand import *
# from rotct import *
# from rotct_rand_defect import *
# from rotct_defect import *


#!/usr/bin/env python
#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Wei Xie
# Last updated: 2014-04-21
##############################################################
import sys
import getopt 
import os
import shutil
import errno
import numpy as np
from scipy import stats

import pymatgen as mg
from pymatgen.analysis import ewald
from pymatgen.io import vaspio

def CenterMassnReflectionCorr(relax,defect): 
    # Correct periodically reflected atoms according to the perfect cell.
    # relax is the CONTCAR (after relaxation) and defect is the POSCAR (before relaxation).
    shift = np.array([0,0,0])
    for ele in relax.keys():
        for i in range(len(relax[ele])):
            for j in range(3):
                shift[j] = shift[j] + relax[ele][i][j] - defect[ele][i][j]
    #shift = shift/len(mg.read_structure('POSCAR_defect'))
    shift = shift/relax.composition.num_atoms 
            
    for ele in relax.keys(): 
        for i in range(len(relax[ele])):
            for j in range(3):
                relax[ele][i][j]=relax[ele][i][j]-shift[j]
                if abs(relax[ele][i][j]-defect[ele][i][j])>abs(relax[ele][i][j]+defect[ele][i][j]-1):
                    if abs(relax[ele][i][j]-defect[ele][i][j]-1)<abs(relax[ele][i][j]-defect[ele][i][j]+1):
                        relax[ele][i][j] = relax[ele][i][j] - 1
                    else:
                        relax[ele][i][j] = relax[ele][i][j] + 1
    return relax

def delete(supercell,chunk):
    for ele in supercell.keys():
        total=len(supercell[ele])
        i=0
        while i<total:
            for n in range(len(chunk[ele])):
                if abs(supercell[ele][i][0] - chunk[ele][n][0])<1e-6 and abs(supercell[ele][i][1] - chunk[ele][n][1])<1e-6 and abs(supercell[ele][i][2] - chunk[ele][n][2])<1e-6:
                    supercell[ele].remove(supercell[ele][i]) # Removing the small cell from supercell.  
                    i = i - 1            
                    total=total-1
            i = i + 1
                                                 
    return supercell

def embed(supercell,relax):
    for ele in relax.keys():
        for i in range(len(relax[ele])):
            try: supercell[ele]
            except:  supercell[ele] = []
            supercell[ele].append(relax[ele][i])
    return supercell
    
def rescale(struct,size,Size): 
    # Necessary for fractional coordinates.
    for ele in struct.keys():
        for i in range(len(struct[ele])):
            for k in range(3):
                struct[ele][i][k] = struct[ele][i][k]*size[k]/Size[k]
    return struct



from MAST.structopt.tools import get_best
import random

def cost(pop, nkeep, Optimizer):
	"""Selection function to order and select structures based on simple relative cost
	"""
	pop = get_best(pop,len(pop))
	newpop = []
	fitnesses = [ind.fitness for ind in pop]
	prob = []
	cumprob = []
	try:
		norms = [fit - pop[nkeep+1].fitness for fit in fitnesses[0:nkeep]]
	except:
		norms = [fit - pop[nkeep-1].fitness for fit in fitnesses[0:nkeep]]
	sumn = sum(norms)
	for i in range(nkeep):
		prob.append(norms[i] / sumn)
		cumprob.append(sum(prob))
	prevcounter = []
	for i in range(nkeep):
		rand = random.random()
		counter = 0
		while cumprob[counter] < rand:
			counter += 1
		if counter in prevcounter:
			a = random.choice(pop)
			newpop.append(a)
			prevcounter.append(a.index)
		else:
			newpop.append(pop[counter])
			prevcounter.append(counter)
	return newpop


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import time
import shutil
import logging
import inspect
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import dirutil
from MAST.utility import Metadata
from MAST.utility import loggerutils
from MAST.ingredients.errorhandler import masterrorhandlers 
from MAST.ingredients.errorhandler import mastvasperrorhandlers 
from pymatgen.core.structure import Structure
from pymatgen.io.vaspio import Poscar
from pymatgen.io.cifio import CifParser
class BaseError(MASTObj):
    """Base error handling class. This class switches between
        program-specific error-handling functions.
        For example, phon-related functions or vasp-related
        functions get their own error class.
    """
    
    def __init__(self, allowed_keys, **kwargs):
        allowed_keys_base = dict()
        allowed_keys_base.update(allowed_keys) 
        MASTObj.__init__(self, allowed_keys_base, **kwargs)
        self.logger = logging.getLogger(self.keywords['name'])
        self.logger = loggerutils.add_handler_for_recipe(self.keywords['name'], self.logger)
  

    def get_error_handlers(self):
        """Get error handler list from custodian.vasp.handlers.
            This is a dynamic function because the Materials Project is
            continuously adding new handlers.
            Implement additional errors within the specific program type
            e.g. vasperror.py
        """
        handlerlist = list()
        #handlerlist = inspect.getmembers(handlers)
        handlerlist.extend(inspect.getmembers(masterrorhandlers))
        handlerlist.extend(inspect.getmembers(mastvasperrorhandlers))
        handlerdict=dict()
        for htry in handlerlist:
            hname = htry[0]
            if 'ErrorHandler' in hname and not (hname == 'ErrorHandler'):
                handlerdict[hname]=htry[1]
        return handlerdict
    
    def set_handler_inputs(self):
        """We may want to override the files that are being checked for
            each specific program type (vasp, phon, generic, etc.)
            Implement additional errors within the specific program type
            e.g. vasperror.py
        """
        handler_input_d=dict()
        handler_input_d['MASTMemoryErrorHandler']=[self.keywords['name'],self.keywords]
        return handler_input_d

    def loop_through_errors(self):
        """Loop through all errors in the error handlers.
        """
        mydir = self.keywords['name']
        errct = 0
        handlerdict = self.get_error_handlers() 
        handler_input_d = self.set_handler_inputs()
        myerror = ""
        os.chdir(mydir)
        self.logger.info("Checking errors for: %s" % mydir)
        for hname in handlerdict.keys():
            hinputs=""
            if hname in handler_input_d.keys():
                hinputs = handler_input_d[hname]
            else: #ignore errors whose inputs are not listed
                self.logger.warning("Skipping error check %s because no input was specified." % hname)
                continue 
            #if hinputs == "mast_skip":
            #    self.logger.info("Skipping %s" % hname)
            #    pass
            self.logger.info("Checking for %s with inputs %s." % (hname, hinputs))
            if len(hinputs) == 0:
                myerror = handlerdict[hname]()
            elif len(hinputs) == 1:
                myerror = handlerdict[hname](hinputs[0])
            elif len(hinputs) == 2:
                myerror = handlerdict[hname](hinputs[0],hinputs[1])
            elif len(hinputs) == 3:
                myerror = handlerdict[hname](hinputs[0],hinputs[1],hinputs[2])
            elif len(hinputs) == 4:
                myerror = handlerdict[hname](hinputs[0],hinputs[1],hinputs[2],hinputs[3])
            elif len(hinputs) == 5:
                myerror = handlerdict[hname](hinputs[0],hinputs[1],hinputs[2],hinputs[3],hinputs[4])
            else:
                raise MASTError(self.__class__.__name__, "Error %s has too many inputs (> 5)" % hname)
            if myerror.check():
                self.logger.error("%s Error found in directory %s!" % (hname, self.keywords['name']))
                errct = errct + 1
                if "mast_auto_correct" in self.keywords['program_keys'].keys():
                    if str(self.keywords['program_keys']['mast_auto_correct']).strip()[0].lower() == 'f':
                        self.logger.error("mast_auto_correct is false for this ingredient. Skipping correction.")
                        continue
                c_dict = myerror.correct()
                self.logger.error("Errors: %s" % c_dict["errors"])
                self.logger.error("Actions taken for %s: %s" % (self.keywords['name'],c_dict["actions"]))
            else:
                self.logger.info("%s No error found." % hname)
                pass
        return errct

    def clean_up_directory(self, alist=list()):
        """Clean up old output files. (Custodian handlers to not necessarily
            remove the old OUTCAR and OSZICAR files.)
            IMPORTANT: Any output to be archived must have been archived by
                the error handlers.
            Args:
                alist <list of str>: List of file names to remove
        """
        for aitem in alist:
            apath = os.path.join(self.keywords['name'], aitem)
            if os.path.isfile(apath):
                self.logger.info("Removing old file %s" % apath)
                os.remove(apath)
        return



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import sys
import fnmatch
import time
from MAST.utility import MASTError
from MAST.utility.metadata import Metadata

def immediate_subdirs(existdir, verbose=0):
    """Walk through directory and return immediate subdirectories
        (only one level)
        as directory names (not full paths)
        Args:
            existdir <str>: Directory, which should exist
            verbose <int>: 0 - concise (default)
                           1 - verbose
    """
    if not (os.path.exists(existdir)):
        raise MASTError("utility","No directory at " +existdir)
    listdir = os.listdir(existdir)
    subdirs=list()
    for myentry in listdir:
        trydir = os.path.join(existdir, myentry)
        if os.path.isdir(trydir):
            subdirs.append(myentry)
    subdirs.sort()
    if verbose == 1:
        print "subdirectories:", subdirs
    return subdirs


def walkdirs(existdir, mindepth=1, maxdepth=5, matchme=""):
    """Walk through directory and return list of subdirectories."""
    if not(os.path.exists(existdir)):
        raise MASTError("utility","No directory at " +existdir)
#   walk and make main list
    walkme=""
    walkentry=""
    walkme = os.walk(existdir)
    if walkme == ():
        raise MASTError("utility walkdirs","No folders found in " + existdir)
    bigdirlist=[]
    for walkentry in walkme:
        bigdirlist.append(walkentry[0])
#   sort list and pare down according to min/max depth
    numsep=0
    minnumsep=0
    maxnumsep=0
    numsep = existdir.count('/') # find base number of forward slashes
    minnumsep = numsep + mindepth
    maxnumsep = numsep + maxdepth
    onedir=""
    onenumsep=0
    smalldirlist=[]
    for onedir in bigdirlist:
        onenumsep = onedir.count('/')
        if (onenumsep >= minnumsep) and (onenumsep <= maxnumsep):
            smalldirlist.append(onedir)
    smalldirlist.sort()
    if not matchme == "":
        matchdirlist=[]
        for mydir in smalldirlist:
            if fnmatch.fnmatch(mydir, matchme):
                matchdirlist.append(mydir)
        return matchdirlist
    else:
        return smalldirlist

def walkfiles(existdir, mindepth=1, maxdepth=5, matchme=""):
    """Walk through directory and subdirectories and return list of files.
        Args:
            existdir <str>: directory under which to search.
            mindepth <int>: minimum depth of folders to search;
                            default 1 = within that directory
            maxdepth <int>: maximum depth of folders to search; default 5
            matchme <str>: string to match; every file ending in matchme
                            will be found, since a * is required at the front
                            to match the full paths.
        Returns:
            <list of str>: list of full file paths
    """
    if not(os.path.exists(existdir)):
        raise MASTError("utility","No directory at " +existdir)
#   walk and make main list
    walkme=""
    walkme = os.walk(existdir)
    if walkme == ():
        raise MASTError("utility walkfiles","No folders found in " + existdir)
    filetree=""
    diritem=""
    fileitem=""
    fullfile=""
    filelist=[]
    filetree = os.walk(existdir)
    for diritem in filetree:
        for fileitem in diritem[2]:
            fullfile = diritem[0] + '/' + fileitem
            filelist.append(fullfile)
    #for fullfile in filelist:
    #    print fullfile
#   #   sort and pare file list
    paredfilelist=[]
    numsep=0
    minnumsep=0
    maxnumsep=0
    numsep = existdir.count('/') # find base number of forward slashes
    minnumsep = numsep + mindepth
    maxnumsep = numsep + maxdepth
    onefile=""
    onenumsep=0
    for onefile in filelist:
        onenumsep = onefile.count('/')
        if (onenumsep >= minnumsep) and (onenumsep <= maxnumsep):
            paredfilelist.append(onefile)
    paredfilelist.sort()
    if not matchme == "":
        matchfilelist=[]
        if not matchme[0] == "*":
            matchme="*"+matchme
        for myfile in paredfilelist:
            if fnmatch.fnmatch(myfile, matchme):
                matchfilelist.append(myfile)
        return matchfilelist
    else:
        return paredfilelist

def get_mast_install_path():
    raise NotImplementedError("MAST_INSTALL_PATH environment variable is no longer being used.")
    getpath = os.getenv('MAST_INSTALL_PATH')
    if getpath == None:
        raise MASTError("utility dirutil","No path set in environment variable MAST_INSTALL_PATH")
    return getpath
def get_mast_scratch_path():
    getpath = os.getenv('MAST_SCRATCH')
    if getpath == None:
        raise MASTError("utility dirutil","No path set in environment variable MAST_SCRATCH")
    return getpath

def get_mast_archive_path():
    getpath = os.getenv('MAST_ARCHIVE')
    if getpath == None:
        raise MASTError("utility dirutil","No path set in environment variable MAST_ARCHIVE")
    return getpath

def get_mast_control_path():
    getpath = os.getenv('MAST_CONTROL')
    if getpath == None:
        raise MASTError("utility dirutil","No path set in environment variable MAST_CONTROL")
    return getpath
def directory_is_locked(dirname):
    if os.path.isfile(dirname + "/mast.write_files.lock"):
        return True
    else:
        return False

def lock_directory(dirname, waitmax=10):
    """Lock a directory using a lockfile.
        Args:
            dirname <str>: Directory name
            waitmax <int>: maximum number of 5-second waits
    """
    if directory_is_locked(dirname):
        wait_to_write(dirname, waitmax)
    lockfile = open(dirname + "/mast.write_files.lock", 'wb')
    lockfile.writelines(time.ctime())
    lockfile.close()

def unlock_directory(dirname, waitmax=10):
    """Unlock a directory by removing the lockfile.
        Args:
            dirname <str>: Directory name
            waitmax <int>: maximum number of 5-second waits
    """
    if directory_is_locked(dirname):
        os.remove(dirname + "/mast.write_files.lock")
    else:
        waitct=1
        okay=0
        while (not directory_is_locked(dirname)) and (waitct <= waitmax):
            if directory_is_locked(dirname):
                os.remove(dirname + "/mast.write_files.lock")
                okay=1
                break
            time.sleep(5)
            waitct=waitct+1
        if not okay==1:
            raise MASTError("utility unlock_directory",
                "Tried to unlock directory %s which was not locked." % dirname)

def wait_to_write(dirname, waitmax=10):
    """Wait to write to directory.
        Args:
            dirname <str>: Directory name
            waitmax <int>: maximum number of 5-second waits
    """
    if waitmax < 1:
        waitmax = 1
    waitcount = 1
    while directory_is_locked(dirname) and (waitcount < waitmax):
        time.sleep(5)
        waitcount = waitcount + 1
    if directory_is_locked(dirname):
        raise MASTError("utility wait_to_write", 
            "Timed out waiting to obtain lock on directory %s" % dirname)
def search_for_metadata_file(metastring="",dirname="", metafilename="metadata.txt", verbose=0):
    """Match a metadata file based on input.
        Args:
            metastring <str>: equals-sign-separated metatag=value pairs with
                                commas separating the meta sections.
                Example: "ingredtype=phonon, neblabel=vac1-vac2, charge=0"
            dirname <str>: directory name to start. Default "" goes to ARCHIVE.
            metafilename <str>: metadata file name. Default "metadata.txt"
            verbose <int>: 1 for verbose messages, 0 otherwise
        Returns:
            dlist <list of str>: list of directories containing matching
                                    metadata files.
    """
    if dirname=="":
        dirname = get_mast_archive_path()
    allmetas = walkfiles(dirname, 1, 5, metafilename)
    if len(allmetas) == 0:
        raise MASTError("utility dirutil, search_for_metadata_file", "No matching metafiles found in %s for tags %s." % (dirname, metastring))
    metaparse=dict()
    metasplit = metastring.split(",")
    for metaitem in metasplit:
        onesplit=metaitem.strip().split("=")
        metaparse[onesplit[0].strip()]=onesplit[1].strip()
    metamatch=list()
    mustmatch=len(metaparse.keys())
    for mtry in allmetas:
        mokay=0
        mymeta = Metadata(metafile=mtry)
        for metatag,metaval in metaparse.iteritems():
            searchresult = mymeta.search_data(metatag)
            if searchresult[1] == metaval:
                mokay=mokay+1
            else:
                if verbose == 1:
                    print metatag, searchresult
        if mokay == mustmatch:
            metamatch.append(mtry)
    if verbose==1:
        print allmetas
        print metaparse
        print metamatch
    return metamatch

def list_methods(myclass=None, printout=1):
    """List the methods in a class.
        Args:
            myclass <Class>: Class, like BaseIngredient
            printout <int>: Print the list to screen
                            1 - print (default)
                            0 - do not print to screen
    """
    import inspect
    mylist=inspect.getmembers(myclass, predicate=inspect.ismethod)
    parsedlist=list()
    for myentry in mylist:
        parsedlist.append(myentry[0])
    if printout > 0:
        for myentry in parsedlist:
            print myentry
    return parsedlist

def dir_is_in_scratch(mydir):
    """Check if the directory is in the $MAST_SCRATCH directory.
        Args:
            mydir <str>: directory name (not full path)
        Returns:
            True or False
    """
    scratch = get_mast_scratch_path()
    dirs_in_scratch = os.listdir(scratch)
    if (mydir in dirs_in_scratch):
        return True
    else:
        return False
def get_test_dir(testname):
    """Get testing directory name.
        Args:
            testname <str>: test directory name, e.g. test_mod_1
        Returns:
            <str>: full test directory, depending on where python is
                    installed and where the tests are located, e.g.
        <python install dir>/lib/python2.7/site-packages/MAST/test/test_mod_1
        where <python install dir> might be:
                //home/username/.local/
                //share/apps/EPD_64bit
                or some other directory
        Assumes that the user starts running the nosetests in the MAST
        directory or in MAST/test
    """
    testdir=""
    curdir = os.getcwd()
    curdir = os.path.normpath(curdir)
    if os.path.basename(curdir) == testname:
        return curdir
    testdir = curdir
    if os.path.basename(curdir) == "MAST":
        testdir = os.path.join(testdir, "test")
    testdir = os.path.join(testdir, testname)
    return testdir



import os
from ase import Atom, Atoms
from MAST.structopt.inp_out.write_xyz import write_xyz
from MAST.structopt.tools.setup_calculator import setup_calculator
from MAST.structopt.tools.find_defects import find_defects
from MAST.structopt.tools.check_cell_type import check_cell_type
from MAST.structopt.fingerprinting import get_fingerprint
from MAST.structopt.tools.lammps import LAMMPS
from ase.calculators.vasp import Vasp
from ase.optimize import BFGS
from ase.units import GPa
from ase.calculators.neighborlist import NeighborList
import numpy
try:
    from mpi4py import MPI
except ImportError:
    pass
import logging
import pdb
import shutil

def eval_energy(Optimizer, individ):
    """Function to evaluate energy of an individual
    Inputs:
        input = [Optimizer class object with parameters, Individual class structure to be evaluated]
    Outputs:
        energy, bul, individ, signal
        energy = energy of Individual evaluated
        bul = bulk structure of Individual if simulation structure is Defect
        individ = Individual class structure evaluated
        signal = string of information about evaluation
    """
    #logger = initialize_logger(Optimizer.loggername)
    logger = logging.getLogger(Optimizer.loggername)
    if 'MAST' in Optimizer.calc_method:
        energy = individ.energy
        bul = individ.bulki
        signal = 'Received MAST structure\n'
        logger.info('Received individual index = {0} from MAST with energy {1}. Returning with no evaluation'.format(
            individ.index, individ.energy))
    else:
        if Optimizer.parallel: 
            rank = MPI.COMM_WORLD.Get_rank()
        logger.info('Received individual HI = {0} with energy {1} for energy evaluation'.format(
            individ.history_index, individ.energy))
        STR='----Individual ' + str(individ.history_index)+ ' Optimization----\n'
        indiv=individ[0]
        if 'EE' in Optimizer.debug:
            debug = True
        else:
            debug = False
        if debug: 
            write_xyz(Optimizer.debugfile,indiv,'Received by eval_energy')
            Optimizer.debugfile.flush()
            logger.debug('Writing recieved individual to debug file')
        # Establish individual structure for evaluation.  Piece together regions when necessary.
        if Optimizer.structure=='Defect':
            indi=indiv.copy()
            bulk=individ.bulki
            nat=indi.get_number_of_atoms()
            if debug: 
                logger.info('Extending defect structure to include bulk len(r1+r2)={0} len(bulk)={1}'.format(nat,len(bulk)))
            csize=bulk.get_cell()                                                                                                         
            totalsol=Atoms(cell=csize, pbc=True)
            totalsol.extend(indi)
            totalsol.extend(bulk)
            for sym,c,m,u in Optimizer.atomlist:
                nc=len([atm for atm in totalsol if atm.symbol==sym])
                STR+='Defect configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n'
        elif Optimizer.structure=='Surface':
            totalsol=Atoms()
            totalsol.extend(indiv)
            nat=indiv.get_number_of_atoms()
            totalsol.extend(individ.bulki)
            if debug:
                logger.info('Extending surface structure to include bulk len(r1+r2)={0} len(bulk)={1}'.format(nat,len(individ.bulki)))
            for sym,c,m,u in Optimizer.atomlist:
                nc=len([atm for atm in totalsol if atm.symbol==sym])
                STR+='Surface-Bulk configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n'
            cell=numpy.maximum.reduce(indiv.get_cell())
            totalsol.set_cell([cell[0],cell[1],500])
            totalsol.set_pbc([True,True,False])
        elif Optimizer.structure=='Cluster':
            totalsol = indiv.copy()
            nat = len(totalsol)
            if debug:
                logger.info('Extending cluster with {0} atoms to center of evaluation box of size {1}'.format(nat,Optimizer.large_box_size))
            origcell = indiv.get_cell()
            totalsol.set_cell([Optimizer.large_box_size,Optimizer.large_box_size,Optimizer.large_box_size])
            totalsol.translate([Optimizer.large_box_size/2.0,Optimizer.large_box_size/2.0,Optimizer.large_box_size/2.0])
        elif Optimizer.structure=='Crystal':
            totalsol = indiv.copy()
            nat = len(totalsol)
        else:
            print 'WARNING: In EvalEnergy. Optimizer.structure not recognized'
            logger.warning('Optimizer.structure not recognized')
        
        # Check for atoms that are too close or out of constrained location
        if Optimizer.constrain_position:
            if Optimizer.structure=='Defect':
                if debug:
                    logger.info('Constraining positions of defect')
                totalsol, stro = constrain_positions(totalsol, Optimizer.solidbulk, Optimizer.sf)
                if debug:
                    logger.info(stro)
                STR+=str0
        min_len=0.7
        if not Optimizer.fixed_region:
            if debug:
                logger.info('Running check minimum distance')
            totalsol, STR = check_min_dist(totalsol, Optimizer.structure, nat, min_len, STR)
            if debug:
                write_xyz(Optimizer.debugfile,totalsol,'After minlength check')
                Optimizer.debugfile.flush()
                logger.debug('Writing individual after checking minimum length')
        
        # Set calculator to use to get forces/energies
        if Optimizer.parallel:
            calc = setup_calculator(Optimizer)
            if Optimizer.fixed_region:
                if debug:
                    logger.info('Setting up fixed region calculator')
                pms=copy.deepcopy(calc.parameters)
                try:
                    pms['mass'][len(pms['mass'])-1] += '\ngroup RO id >= {0}\nfix freeze RO setforce 0.0 0.0 0.0\n'.format(nat)
                except KeyError:
                    pms['pair_coeff'][0] += '\ngroup RO id >= {0}\nfix freeze RO setforce 0.0 0.0 0.0\n'.format(nat)
                calc = LAMMPS(parameters=pms, files=calc.files, keep_tmp_files=calc.keep_tmp_files, tmp_dir=calc.tmp_dir)
                lmin = copy.copy(Optimizer.lammps_min)
                if debug:
                    logger.info('Setting up no local minimization calculator')
                Optimizer.lammps_min = None
                Optimizer.static_calc = setup_calculator(Optimizer)
                Optimizer.lammps_min = lmin
        else:
            calc=Optimizer.calc
        totalsol.set_calculator(calc)
        totalsol.set_pbc(True)
        
        # Perform Energy Minimization
        if not Optimizer.parallel:
            if debug: 
                write_xyz(Optimizer.debugfile,totalsol,'Individual sent to Energy Minimizer')
                logger.debug('Writing structure sent to energy minimizer')
        try:
            cwd = os.getcwd()
            if Optimizer.ase_min == True:
                if debug:
                    logger.info('Running ASE minimizer')
                if Optimizer.calc_method=='LennardJones':
                    logger.warn('Must run ase LJ calculator with pbc=False')
                    totalsol.set_pbc(False)
                totalsol, energy, pressure, volume, STR = run_ase_min(totalsol, Optimizer.ase_min_fmax, Optimizer.ase_min_maxsteps, Optimizer.fitness_scheme, STR)
            else:
                if debug:
                    logger.info('Running local energy calculator')
                if Optimizer.fixed_region:
                    totalsol, energy, pressure, volume, STR = run_energy_eval(totalsol, Optimizer.calc_method, Optimizer.fixed_region, Optimizer.fitness_scheme, STR, Optimizer.static_calc)
                else:
                    totalsol, energy, pressure, volume, STR = run_energy_eval(totalsol, Optimizer.calc_method, False, Optimizer.fitness_scheme, STR)
        except Exception, e:
            logger.critical('Error in energy evaluation: {0}'.format(e), exc_info=True)
            path = os.path.join(cwd,'TroubledLammps')
            if not os.path.exists(path):
                os.mkdir(path)
            #Copy files over
            shutil.copyfile(calc.trajfile,os.path.join(path,os.path.basename(calc.trajfile)))
            shutil.copyfile(calc.infile,os.path.join(path,os.path.basename(calc.infile)))
            shutil.copyfile(calc.logfile,os.path.join(path,os.path.basename(calc.logfile)))
            shutil.copyfile(calc.datafile,os.path.join(path,os.path.basename(calc.datafile)))
            raise RuntimeError('{0}:{1}'.format(Exception,e))
        if not Optimizer.parallel:
            if debug:
                write_xyz(Optimizer.debugfile,totalsol,'Individual after Energy Minimization')
                Optimizer.debugfile.flush()
                logger.debug('Writing structure recieved from energy minimizer')
       
        # Separate structures into distinct pieces
        if Optimizer.structure=='Defect':
            if Optimizer.fixed_region==True or Optimizer.finddefects==False:
                if debug:
                    logger.info('Identifying atoms in defect structure based on ID')
                individ[0]=totalsol[0:nat]
                bul=totalsol[(nat):len(totalsol)]
                individ[0].set_cell(csize)
            else:
                if debug:
                    logger.info('Applying find defects scheme to identify R1 and R2 for Defect')
                if 'FD' in Optimizer.debug:
                    outt=find_defects(totalsol,Optimizer.solidbulk,Optimizer.sf,atomlistcheck=Optimizer.atomlist,trackvacs=Optimizer.trackvacs,trackswaps=Optimizer.trackswaps,debug=Optimizer.debugfile)
                else:
                    outt=find_defects(totalsol,Optimizer.solidbulk,Optimizer.sf,atomlistcheck=Optimizer.atomlist,trackvacs=Optimizer.trackvacs,trackswaps=Optimizer.trackswaps,debug=False)
                individ[0]=outt[0]
                bul=outt[1]
                individ.vacancies = outt[2]
                individ.swaps = outt[3]
                STR += outt[4]
            indiv=individ[0]
        elif Optimizer.structure=='Surface':
            if debug:
                logger.info('Finding surface top layer')
            top,bul=find_top_layer(totalsol,Optimizer.surftopthick)
            indiv=top.copy()
            individ[0]=top.copy()
            bul = Atoms()
        elif Optimizer.structure=='Crystal':
            if debug:
                logger.info('Checking crystal cell type')
            celltype = check_cell_type(totalsol)
            STR+='Cell structure = {0}\n'.format(celltype)
            bul = Atoms()
            individ[0] = totalsol.copy()
        elif Optimizer.structure=='Cluster':
            volume = get_cluster_volume(totalsol)
            bul = Atoms()
            if debug:
                logger.info('Translating cluster back to smaller box size location')
            totalsol.translate([-Optimizer.large_box_size/2.0,-Optimizer.large_box_size/2.0,-Optimizer.large_box_size/2.0])
            totalsol.set_cell(origcell)
            individ[0] = totalsol.copy()
        
        # Add concentration energy dependence
        if Optimizer.forcing=='energy_bias':
            if debug:
                logger.info('Applying energy bias for atoms with different number of atoms of type than in atomlist')
            n=[0]*len(Optimizer.atomlist)
            for i in range(len(Optimizer.atomlist)):
                n[i]=len([inds for inds in totalsol if inds.symbol==Optimizer.atomlist[i][0]])
                n[i]=abs(n[i]-Optimizer.atomlist[i][1])
            factor=sum(n)**3
            energy=(energy+factor)/totalsol.get_number_of_atoms()
            STR+='Energy with Bias = {0}\n'.format(energy)
        elif Optimizer.forcing=='chem_pot':
            if debug:
                logger.info('Applying chemical potential bias for atoms with different number of atoms of type than in atomlist')
            n=[0]*len(Optimizer.atomlist)
            for i in range(len(Optimizer.atomlist)):
                n[i]=len([inds for inds in totalsol if inds.symbol==Optimizer.atomlist[i][0]])
                n[i]=n[i]*Optimizer.atomlist[i][3]
            factor=sum(n)
            energy=(energy+factor)/totalsol.get_number_of_atoms()
            STR+='Energy with Chemical Potential = {0}\n'.format(energy)

        individ.energy=energy
        individ.buli=bul
        individ.pressure=pressure
        individ.volume=volume
    
        if Optimizer.fingerprinting:
            if debug:
                logger.info('Identifying fingerprint of new structure')
            individ.fingerprint=get_fingerprint(Optimizer,individ,Optimizer.fpbin,Optimizer.fpcutoff)
        if Optimizer.parallel:
            calc.clean()
            signal = 'Evaluated individual {0} on {1}\n'.format(individ.index,rank)
            signal +=STR
        else:
            signal=STR

    return energy, bul, individ, signal

def constrain_positions(indiv, bulk, sf):
    STR=''
    ts = indiv.copy()
    indc,indb,vacant,swap,stro = find_defects(ts,bulk,0)
    sbulk = bulk.copy()
    bcom = sbulk.get_center_of_mass()
    #totalsol.translate(-bulkcom)
    #indc.translate(-bulkcom)
    #totalsol.append(Atom(position=[0,0,0]))
    #             for one in indc:
    #                 index = [atm.index for atm in totalsol if atm.position[0]==one.position[0] and atm.position[1]==one.position[1] and atm.position[2]==one.position[2]][0]
    #                 if totalsol.get_distance(-1,index) > Optimizer.sf:
    #                     r = random.random()
    #                     totalsol.set_distance(-1,index,Optimizer.sf*r,fix=0)
    #             totalsol.pop()
    #             totalsol.translate(bulkcom)
    com = indc.get_center_of_mass()
    dist = (sum((bcom[i] - com[i])**2 for i in range(3)))**0.5
    if dist > sf:
        STR+='Shifting structure to within region\n'
        r = random.random()*sf
        comv = numpy.linalg.norm(com)
        ncom = [one*r/comv for one in com]
        trans = [ncom[i]-com[i] for i in range(3)]
        indices = []
        for one in indc:
            id = [atm.index for atm in totalsol if atm.position[0]==one.position[0] and atm.position[1]==one.position[1] and atm.position[2]==one.position[2]][0]
            ts[id].position += trans
    return ts, STR

def check_min_dist(totalsol, type='Defect', nat=None, min_len=0.7, STR=''):
    if type=='Defect' or type=='Crystal' or type=='Surface':
        if nat==None:
            nat=len(totalsol)
        cutoffs=[2.0 for one in totalsol]
        nl=NeighborList(cutoffs,bothways=True,self_interaction=False)
        nl.update(totalsol)
        for one in totalsol[0:nat]:
            nbatoms=Atoms()
            nbatoms.append(one)
            indices, offsets=nl.get_neighbors(one.index)
            for index, d in zip(indices,offsets):
                index = int(index)
                sym=totalsol[index].symbol
                pos=totalsol[index].position + numpy.dot(d,totalsol.get_cell())
                at=Atom(symbol=sym,position=pos)
                nbatoms.append(at)
            while True:
                dflag=False
                for i in range(1,len(nbatoms)):
                    d=nbatoms.get_distance(0,i)
                    if d < min_len:
                        nbatoms.set_distance(0,i,min_len+.01,fix=0.5)
                        STR+='--- WARNING: Atoms too close (<0.7A) - Implement Move ---\n'
                        dflag=True
                if dflag==False:
                    break
            for i in range(len(indices)):
                totalsol[indices[i]].position=nbatoms[i+1].position
            totalsol[one.index].position=nbatoms[0].position
            nl.update(totalsol)
    elif type=='Cluster':
        for i in range(len(totalsol)):
            for j in range(len(totalsol)):
                if i != j:
                    d=totalsol.get_distance(i,j)
                    if d < min_len:
                        totalsol.set_distance(i,j,min_len,fix=0.5)
                        STR+='--- WARNING: Atoms too close (<0.7A) - Implement Move ---\n'
    else:
        print 'WARNING: In Check_Min_Dist in EvalEnergy: Structure Type not recognized'
    return totalsol, STR

def get_cluster_volume(cluster):
    max_pos = numpy.maximum.reduce(cluster.get_positions())
    min_pos = numpy.minimum.reduce(cluster.get_positions())
    diff_pos = [max_pos[i]-min_pos[i] for i in range(3)]
    vol = diff_pos[0]*diff_pos[1]*diff_pos[2]
    return vol

def run_ase_min(totalsol, fmax=0.01, mxstep=1000, fitscheme='totalenfit', STR=''):
    try:
        dyn=BFGS(totalsol)
        dyn.run(fmax=fmax, steps=mxstep)
    except OverflowError:
        STR+='--- Error: Infinite Energy Calculated - Implement Random shake---\n'
        totalsol.rattle(stdev=0.3)
        dyn=BFGS(totalsol)
        dyn.run(fmax=fmax, steps=mxstep)
    except numpy.linalg.linalg.LinAlgError:
        STR+='--- Error: Singular Matrix - Implement Random shake ---\n'
        totalsol.rattle(stdev=0.2)
        dyn=BFGS(totalsol)
        dyn.run(fmax=fmax, steps=mxstep)
    # Get Energy of Minimized Structure
    en=totalsol.get_potential_energy()
    if fitscheme == 'enthalpyfit':
        pressure=totalsol.get_isotropic_pressure(totalsol.get_stress())
    else:
        pressure=0
    volume = totalsol.get_volume()
    energy=en
    return totalsol, energy, pressure, volume, STR

def run_energy_eval(totalsol, calc_method='LAMMPS', fx_region=False, fit_scheme='totalenfit', STR='', static_calc=None):
    if calc_method=='VASP':
        en=totalsol.get_potential_energy()
        calcb=Vasp(restart=True)
        totalsol=calcb.get_atoms()
        stress=calcb.read_stress()
    else:
        totcop = totalsol.copy()
        OUT = totalsol.calc.calculate(totalsol)
        totalsol = OUT['atoms']
        totalsol.set_pbc(True)
        if fx_region:
            STR+='Energy of fixed region calc = {0}\n'.format(OUT['thermo'][-1]['pe'])
            totalsol.set_calculator(static_calc)
            OUT=totalsol.calc.calculate(totalsol)
            totalsol=OUT['atoms']
            totalsol.set_pbc(True)
            STR+='Energy of static calc = {0}\n'.format(OUT['thermo'][-1]['pe'])
        en=OUT['thermo'][-1]['pe']
        stress=numpy.array([OUT['thermo'][-1][i] for i in ('pxx','pyy','pzz','pyz','pxz','pxy')])*(-1e-4*GPa)
    if fit_scheme == 'enthalpyfit':
        pressure = totalsol.get_isotropic_pressure(stress)
    else:
        pressure = 0
    volume = totalsol.get_volume()
    energy=en
    STR+='Energy per atom = {0}\n'.format(energy/len(totalsol))
    return totalsol, energy, pressure, volume, STR



from MAST.structopt.inp_out.write_xyz import write_xyz
import logging
import pdb

def crossover_switch(child1, child2, Optimizer):
    """Functions for selecting and pairing individuals 
    for crossovers"""
    #logger = initialize_logger(Optimizer.loggername)
    logger = logging.getLogger(Optimizer.loggername)
    if 'CX' in Optimizer.debug:
        debug = True
    else:
        debug = False
    if debug:
        s1 = child1[0].copy()
        s2 = child2[0].copy()
        if Optimizer.structure=='Defect':
            s1.extend(child1.bulki.copy())
            s2.extend(child2.bulki.copy())
        write_xyz(Optimizer.debugfile,s1,'First Cx Individual - Pre ')
        write_xyz(Optimizer.debugfile,s2,'Second Cx Individual - Pre')
    passflag = True
    scheme = Optimizer.cx_scheme
    try:
        exec "from MAST.structopt.crossover.{0} import {0}".format(scheme)
        nchild1, nchild2 = eval('{0}(child1, child2, Optimizer)'.format(scheme))
    except NameError, e:
        logger.warning('Specified Crossover not one of the available options. Please check documentation and spelling! Crossover : {0}. {1}'.format(Optimizer.cx_scheme,e), exc_info=True)
        print 'Name Error:', e
        passflag = False
    except Exception, e:
        logger.error('Error in Crossover fuction : {0}. {1}'.format(Optimizer.cx_scheme,e), exc_info=True)
        print 'Exception: ', e
        passflag = False
    if not passflag:
        try:
            from MAST.structopt.crossover.cxtp import cxtp
            nchild1, nchild2 = cxtp(child1, child2, Optimizer)
            passflag = True
        except NameError, e:
            logger.error('Error attempting to load back up cxTP Crossover.  Something very wrong!! {1}'.format(e), exc_info=True)
            print 'CXTP Name Error: ', e
        except Exception, e:
            logger.error('Error in cxtp Crossover {0}'.format(e), exc_info=True)
            print 'Exception: ',e
    if passflag:
        child1=nchild1
        child2=nchild2
        child1.energy=0
        child2.energy=0
        child1.fitness=0
        child2.fitness=0 
        ch1hi=child1.history_index
        child1.history_index = '(' + repr(child1.index) + '+' + repr(child2.index) + ')'
        child2.history_index = '(' + repr(child2.index) + '+' + repr(child1.index) + ')'
    if debug:
        s1 = child1[0].copy()
        s2 = child2[0].copy()
        if Optimizer.structure=='Defect':
            s1.extend(child1.bulki.copy())
            s2.extend(child2.bulki.copy())
        write_xyz(Optimizer.debugfile,s1,'First Cx Individual - Post')
        write_xyz(Optimizer.debugfile,s2,'Second Cx Individual - Post')
    
    return child1, child2
	


import os
try:
    from mpi4py import MPI
except:
    pass

def ofile(name):
    l=open(name,'a')
    return l

def setup_par_output(filename, restart, nindiv, indiv_defect_write, genealogy, 
    allenergyfile, fingerprinting, debug):
    """
        Subprogram to set up the directories and file outputs for the optimizer.
        Inputs:
            filename - String name of the run output folder
            restart - True/False boolean to determine if reading from 
                previous output folder
            nindiv - Integer number of structures to generate output
            indiv_defect_write - True/False boolean to determine if iso output 
                files should be created for Defect case
            genealogy - True/False boolean to determine whether or not 
                to create an output file for the genealogy of the run structures
            allenergies - True/False boolean to determine whether or not to 
                create an output file for all the energies of each generation
            fingerprinting - True/False boolean to determine whether or not 
                to create the output files for fingerprint distances and 
                minimum fingerprint by generation
            debug - List of strings.  If debug !=['None'], then a debug 
                file will be created
        Output:
            Dictionary containing output files where the key describes the file 
            structure. Keys include:
                Files = List of file objects for individual structures in a population
                ifiles = List of file objects for isolated sections of individual 
                    structures in a population for a Defect run
                output = File object for general output
                summary = File object for general summary
                Genealogyfile = File object for genealogy data or None
                tenergyfile = File object for all energy data or None
                debugfile = File object for debug file or None
                fpfile = File object for fingerprint distance data or None
                fpminfile = File object for minimum fingerprint data or None
            Note that with the exception of the general output file, all files will
            be stored in a folder under the filename input given
    """
    rank = MPI.COMM_WORLD.Get_rank()
    path = os.path.join(os.getcwd(), '{0}-rank{1}'.format(filename,rank))
    if not os.path.exists(path):
        if restart:
            raise RuntimeError('Cannot find directory for restart', path)
        os.mkdir(path)
    flist = [os.path.join(path,'indiv{0:02d}.xyz'.format(x)) for x in range(nindiv)]
    flist.append(os.path.join(path,'StructureSummary.txt'))
    files = map(ofile,flist)
    if indiv_defect_write:
        iflist = [os.path.join(path,'indiv{0:02d}-iso.xyz'.format(x)) for x in range(nindiv)]
        ifiles = map(ofile,iflist)
    else:
        ifiles = None
    outname = '{0}.txt'.format(filename)
    output = open(outname,'a')
    summary = open(os.path.join(path,'Summary-{0}.txt'.format(filename)),'a')
    if genealogy:
        Genealogyfile = open(os.path.join(path,'Genealogy-{0}.txt'.format(filename)),'a')
    else:
        Genealogyfile = None
    if allenergyfile: 
        tenergyfile = open(os.path.join(path,'All_Energies-{0}.txt'.format(filename)),'a')
    else:
        tenergyfile = None
    if debug != ['None']:
        debugfile = open(os.path.join(path,'Debug.xyz'),'a')
    else:
        debugfile = None
    if fingerprinting: 
        fpfile=open(os.path.join(path,'Fingerprints-{0}.txt'.format(filename)),'a')
        fpminfile=open(os.path.join(path,'FingerprintMin-{0}.txt'.format(filename)),'a')
    else:
        fpfile = None
        fpminfile = None
    outfiles = {'files':files, 'ifiles':ifiles, 'output':output, 'summary':summary,
        'Genealogyfile':Genealogyfile, 'tenergyfile':tenergyfile, 'debugfile':debugfile,
        'fpfile':fpfile, 'fpminfile':fpminfile}
    return outfiles



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import logging
from MAST.utility import loggerutils
from MAST.utility import MASTFile
from MAST.utility import MASTError

"""Utility to read the indented recipe file."""
def read_recipe(filename, verbose=0):
    """Read the indented recipe.
        "Recipe" is a protected keyword signaling the recipe name.
        Args:
            filename <str>: Full path to recipe file
        Returns:
            totpdict <dict>: Dictionary of 
                [parentname][child]=[parent method group]
            totcdict <dict>: Dictionary of
                [childname]['parents']=[parent,parent,...]
                [childname]['method']=[method group]
            rname <str>: Recipe name
    """
    logger=logging.getLogger('mast')
    logger=loggerutils.add_handler_for_control(logger)

    rfile = MASTFile(filename)
    rdata = list()
    #preprocess by removing blank lines and any "recipe" line
    for line in rfile.data:
        if '\t' in line:
            raise MASTError("recipe/recipeutility", "Recipe at %s contains tabs! Please convert all indentations to the appropriate number of groups of four spaces." % filename)
        myline = line.rstrip() #right-hand strip of carriage return only
        if len(myline) == 0: #blank line
            pass
        elif (len(myline) > 6) and (myline[0:6].lower() == "recipe"):
            rname = myline.split()[1]
        elif myline.strip()[0] == "#": #comment line, hash starting wherever
            pass
        else:
            rdata.append(myline)
    subrdict = split_into_subrecipes(rdata) 
    howtorun=dict()
    parentstocheck=dict()
    howtoupdate=dict()
    for subkey in subrdict.keys():
        idict = make_indentation_dictionary(subrdict[subkey])
        if verbose == 1:
            ikeys = idict.keys()
            ikeys.sort()
            for indentkey in ikeys:
                logger.info("Indent: %s" % indentkey)
                for myitem in idict[indentkey]:
                    logger.info(myitem)
        [onehtu, oneptc, onehtr]=parse_indentation_dict(idict)
        for pkey in onehtu.keys():
            if not (pkey in howtoupdate.keys()):
                howtoupdate[pkey]=dict()
            for pmkey in onehtu[pkey].keys():
                howtoupdate[pkey][pmkey]=onehtu[pkey][pmkey]
        for ckey in oneptc.keys():
            if not (ckey in parentstocheck.keys()):
                parentstocheck[ckey]=list()
            parentstocheck[ckey].extend(oneptc[ckey])
            howtorun[ckey]=onehtr[ckey]
    if verbose==1:
        logger.info("How-to-update-children tree: ")
        keylist = howtoupdate.keys()
        keylist.sort()
        for htukey in keylist:
            logger.info("%s: %s" % (htukey, howtoupdate[htukey]))
        clist = parentstocheck.keys()
        clist.sort()
        logger.info("Parents-to-check tree: ")
        for ckey in clist:
            logger.info("%s: %s" % (ckey, parentstocheck[ckey]))
        logger.info("How-to-run tree: ")
        for ckey in clist:
            logger.info("%s: %s" % (ckey, howtorun[ckey]))

    return [howtoupdate, parentstocheck, howtorun, rname]

def split_into_subrecipes(mydata):
    """Split an entire recipe into subrecipes. Each new zero-level 
        indentation starts a new subrecipe.
        Args:
            mydata <list>: List of lines
        Returns:
            subrecipes <dict>: Dictionary of subrecipe lines (lists)
                subrecipe[1]=list()
    """
    subrecipes=dict()
    ridx=0
    for myline in mydata:
        if get_indent_level(myline) == 0:
            ridx = ridx + 1
            subrecipes[ridx]=list()
        subrecipes[ridx].append(myline)
    return subrecipes

def get_indent_level(myline):
    """Get the indentation level of a line.
        Every 4 spaces is an indentation level.
        Args:
            line <str>: Line to get the indentation level of.
        Returns:
            ilevel <int>: Indentation level. 0 = "writing"
                                             1 = "    writing"
                                             2 = "        writing" 
                                             etc.
                            -1 if indentation level was not determined.
    """
    idx=0
    ilevel=0
    mylen = len(myline)
    while idx < mylen:
        if not (myline[idx] == " "):
            if myline[0:idx].strip() == "": #all previous chars are spaces
                return ilevel
        idx = idx + 4
        ilevel = ilevel + 1
    return -1

def parse_for_name_and_instructions(myline):
    """Parse line for name(s) and instruction name(s).
        Args: 
            myline <str>: Line to parse. Should look something like:
            defect1_stat (singlerun_to_phonon)
            OR
            defect1_stat (singlerun_to_neb), defect2_stat (singlerun_to_neb)
        Returns:
            pdict <dict>: Dictionary of name = instructions
    """
    mysplit = myline.split(',')
    pdict=dict()
    for ppiece in mysplit:
        ppiecestr = ppiece.strip()
        ppsplit = ppiecestr.split()
        pname = ppsplit[0]
        if len(ppsplit) > 1:
            pmethod = ppsplit[1][1:-1] #remove parentheses
        else:
            pmethod = "default"
        pdict[pname] = pmethod
    return pdict

def parse_indentation_dict(idict):
    """Parse an indentation dictionary into three parts. 
        Args:
            idict <dict>: Indentation dictionary
        Returns:
            how_to_update <dict>: Dictionary of how to update
                from the parent to the child:
                how_to_update['perfect_opt1']['perfect_opt2']=
                            ['volrelax_to_singlerun']
            parents_to_check <dict>: Dictionary of parents
                for the child to check (as a list)
                parents_to_check['neb_1-2_opt1']=
                        ['defect1_stat','defect2_stat']
            how_to_run <dict>: Dictionary of how to write,
                evaluate run readiness, run, and evaluate
                completion of the ingredient:
                how_to_run['perfect_opt1']=['volrelax_to_singlerun']
    """
    htu=dict()
    ptc=dict()
    htr=dict()
    iidx=0
    ikeys = idict.keys()
    ikeys.sort()
    while iidx in ikeys:
        plen = len(idict[iidx])
        pidx = 0
        while pidx < plen:
            lct = idict[iidx][pidx][0]
            pname = idict[iidx][pidx][1]
            pmethod = idict[iidx][pidx][2]
            if not (pname in htu.keys()):
                htu[pname]=dict()
            if not (iidx + 1) in ikeys: #no children
                pass
            else:
                cllist = idict[iidx+1]
                for clitem in cllist:
                    clnum = clitem[0]
                    clname = clitem[1]
                    clmethod = clitem[2]
                    addme=0
                    if (clnum > lct):
                        if (pidx + 1) >= plen: #no more parents
                            addme=1
                        else:
                            if lct == idict[iidx][pidx+1][0]: #dual parent
                                addme=1
                            elif clnum < idict[iidx][pidx+1][0]:
                                addme=1
                    if addme == 1:
                        htu[pname][clname]=pmethod
                        if not (clname in ptc.keys()):
                            ptc[clname]=list()
                            htr[clname]=clmethod 
                        ptc[clname].append(pname)
            if not (pname in ptc.keys()):
                ptc[pname]=list() #add tree originator(s)
                htr[pname]=pmethod
            pidx=pidx+1
        iidx=iidx+1
    return [htu,ptc,htr]

def make_indentation_dictionary(subrlist):
    """Parse a single subrecipe into an indentation dictionary:
        idict[indentlevel]=list of [linenumber,name,instructions]
        Args:
            subrlist <list>: List of subrecipe lines
        Returns:
            idict <dict>: Dictionary of indentations
    """
    idict=dict() #creates name/instruction dictionaries 
                        #by indentation level
    lct=0
    for line in subrlist:
        lct=lct+1
        indent = get_indent_level(line)
        if not (indent in idict.keys()):
            idict[indent]=list()
        pdict = parse_for_name_and_instructions(line)
        for parent in pdict.keys():
            idict[indent].append([lct, parent, pdict[parent]])
    return idict



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Zhewen Song
# Last updated: 2014-04-25
# Five-frequency model equation from R. E. Howard and J. R. Manning, Physical Review, Vol. 154, 1967.
# Eight-frequency model equation from P. B. Ghate, Physical Review, Vol. 133, 1963.
##############################################################

import sys, getopt, os
import numpy as np
import pymatgen as mg
from MAST.utility import fileutil
  
class ParsingInputFiles(object):
    """Parsing files of energy and frequency.
        Attributes:
            self.inp <str>: input file name
            self.get_item_name <dict of dict>: get directories from input
            self.get_lattice <dict>: lattice information
            self.get_HB_and_HVf <float>: vacancy formation energy or/and binding energy
            self.get_barrier <dict>: energy barrier
            self.get_v <dict>: attempt frequency 
    """
    def __init__(self,inp):
        self.inp = inp
        
    def getinfo(self,line):
        """Splitting a certain line with space and enter.
            Args:
                line <str>: a certain line to be handled
        """
        line=line.strip('\n')
        data=line.split(' ')
        while 1:
            try: data.remove('')
            except ValueError: break
        return data
        
    def get_item_name(self,keyword):
        """Obtaining directories from different items such as E* and H* (the energy), v* (the frequency), frequency type (5 or 8) and lattice parameters.
            Args:
                keyword <str>: keywords that specify the desired items, including 'E', 'H', 'v', 'type', 'lattice'
        """
        item_name = dict()
        content = open(self.inp,'r').readlines()
        Elist=['E0','E1','E2','E3','E4','Ea','Eb','Ec','EX','Eap','Ebp','Ecp','EXp']
        vlist=['v0','v1','v2','v3','v4','va','vb','vc','vX','vap','vbp','vcp','vXp']
        Hlist=['HB','HVf']
        for i in range(len(content)):
            line = self.getinfo(content[i])
            if not len(line)==0:
                if (keyword=='E' and line[0] in Elist) or (keyword=='v' and line[0] in vlist):
                    item_name[line[0]] = []
                    if len(line)==2:
                        try:
                            item_name[line[0]].append(float(line[1]))
                        except ValueError:
                            pass
                    else:
                        for index in range(1,len(line)):
                            item_name[line[0]].append(line[index])
                elif (keyword=='H' and line[0] in Hlist):
                    if line[0] == 'HB':
                        if len(line) > 2:
                            item_name['HB'] = dict()
                            item_name['HB']['perfect'] = line[1]
                            item_name['HB']['vac-sub'] = line[2]
                            item_name['HB']['sub'] = line[3]
                            item_name['HB']['vac'] = line[4]
                        else:
                            item_name['HB'] = float(line[1])
                    elif line[0] == 'HVf':
                        if len(line) > 2:
                            item_name['HVf'] = dict()
                            item_name['HVf']['perfect'] = line[1]
                            item_name['HVf']['vac'] = line[2]
                        else:
                            item_name['HVf'] = float(line[1])
                elif keyword=='temp' and 'temp' in line[0]:
                    item_name['temp'] = [float(line[1]),float(line[2]),float(line[3])]
                elif keyword=='type' and 'type' in line[0]:
                    if line[0]=='type' and line[1]=='fcc_5freq':
                        item_name = 5
                    elif line[0]=='type' and line[1]=='hcp_8freq':  
                        item_name = 8
                elif keyword=='lattice' and 'lattice' in line[0]:
                    item_name = line[1]
                elif keyword=='plotdisplay' and 'plotdisplay' in line[0]:
                    item_name = line[1]
        return item_name
    
    def get_latt(self,model,latt):
        """Obtaining lattice information from the POSCAR.
            Args:
                types <dict>: type of frequency model, types={'types':<int>}
                lattice <str>: specific name of the POSCAR given by the user
        """
        data={'a':0,'c':0,'No.':0}
        os.system('cp '+latt+'_POSCAR POSCAR')
        struct = mg.read_structure('POSCAR')
        os.system('rm POSCAR')
        reduced = mg.symmetry.finder.SymmetryFinder(struct,0.001).get_primitive_standard_structure()   
        data['No.'] = len(struct)
        if model==5:
            data['a'] = reduced.lattice.abc[0]*np.sqrt(2)*10**(-8)
        if model==8:
            data['a'] = reduced.lattice.abc[0]*10**(-8)
            data['c'] = reduced.lattice.abc[2]*10**(-8)
        return data
        
    def get_HB_and_HVf(self,Hdir,numatom,keyword):
        """Obtaining the value of binding energy HB and vacancy formation energy HVf.
            Args:
                Hdir <dict of dict>: Hdir={'HB':{'perfect': *},{'vac-sub': *},{'sub': *},{'vac': *}},'HVf':{{'perfect': *},{'vac': *}}}, * is <file_name> or <value>
                numatom <int>: number of atoms in the system
                keyword <str>: Either 'HB' or 'HVf'
        """
        try: 
            float(Hdir[keyword])
            return Hdir[keyword]
        except TypeError:
            ene = {}
            for key in Hdir[keyword].keys():
                try: 
                    ene[key]=float(Hdir[keyword][key])
                except ValueError: 
		    ene[key] = Hdir[keyword][key]
		    line = open(ene[key]+'_OSZICAR','r').readlines()      
                    pt = -1   
                    while not 'E0' in line[pt]: pt = pt - 1
                    ene[key] = float(self.getinfo(line[pt])[4])
            if keyword=='HVf':
		return ene['vac'] - (numatom - 1)*ene['perfect']/numatom
            elif keyword=='HB': 
		return ene['perfect'] + ene['vac-sub'] - ene['sub'] - ene['vac']
    
    def get_barrier(self,Edir,Edir_saddle,Edir_min):
        """Obtaining the energy barriers.
            Args:
                Edir <list of dict>: Edir={'E0':*,'E1':*,...}, * is the list like [<local_min>,<saddle>]
                Edir_saddle <dict>: Edir_saddle={'E0':<saddle>,...}
                Edir_min <dict>: Edir_min={'E0':<local_min>,...}
        """        
        enebarr = dict()
        enesaddle = dict()
        eneend = dict()
        for freq in Edir.keys():
            if not len(Edir[freq])==1: 
                try: enesaddle[freq]=float(Edir_saddle[freq])
                except ValueError:
                    line = open(Edir_saddle[freq]+'_OSZICAR','r').readlines()
                    pt = -1
                    while not 'E0' in line[pt]: pt = pt - 1
                    enesaddle[freq] = float(self.getinfo(line[pt])[4])
	for freq in Edir.keys():
            if not len(Edir[freq])==1: 
                try: eneend[freq]=float(Edir_min[freq])
                except ValueError:
                    line = open(Edir_min[freq]+'_OSZICAR','r').readlines()
                    pt = -1
                    while not 'E0' in line[pt]: pt = pt - 1
                    eneend[freq] = float(self.getinfo(line[pt])[4])
	for freq in Edir.keys():
            if len(Edir[freq])==1: enebarr[freq] = Edir[freq][0]
            else: enebarr[freq] = enesaddle[freq] - eneend[freq] 
	return enebarr
    
    def get_v(self,vdir,vdir_num,vdir_denom):
        """Obtaining the attempt frequencies.
            Args:
                vdir <list of dict>: vdir={'v0':*,'v1':*,...}, * is the list like [<local_min>,<saddle>]
                vdir_num <dict>: vdir_num={'E0':<local_min>,...}
                Edir_denum <dict>: Edir_denom={'E0':<saddle>,...}
        """    
        v = dict()
        v_num = dict()
        v_denom = dict()
        for freq in vdir.keys():
            if len(vdir[freq])==1: v[freq] = vdir[freq][0]
            else:
                if os.path.isfile(vdir_num[freq]+'_FREQ') and os.path.isfile(vdir_num[freq]+'_FREQ'): # Reading data from FREQ files
                    fn=open(vdir_num[freq]+'_FREQ','r')
                    nthzlist=self.getinfo(fn.readline())
		    i=0
		    num_num=0
                    v_num[freq]=1.0
                    while i<len(nthzlist):
                        if abs(float(nthzlist[i]))>0.1: 
			    v_num[freq]=v_num[freq]*float(nthzlist[i])
		  	    num_num+=1
                        i+=1
		    fd=open(vdir_denom[freq]+'_FREQ','r')
                    dthzlist=self.getinfo(fd.readline())
		    j=0
		    denom_num=0
                    v_denom[freq]=1.0
                    while j<len(dthzlist):           
                        if abs(float(dthzlist[j]))>0.1: 
			    v_denom[freq]=v_denom[freq]*float(dthzlist[j])
			    denom_num+=1
                        j+=1
		    if not num_num==denom_num+1: 
                        print 'WARNING: Numbers of non-zero frequencies at local minimum and at saddle point do not match! Please double-check the FREQ files of %s and %s'%(vdir_num[freq],vdir_denom[freq])

                else: # Reading data from OUTCAR files
        	    nthzlist = fileutil.grepme(vdir_num[freq]+'_OUTCAR', "2PiTHz")
                    num_num=len(nthzlist)
		    dthzlist = fileutil.grepme(vdir_denom[freq]+'_OUTCAR', "2PiTHz")
           	    denom_num=len(dthzlist)
           	    im_num=0
           	    im_denom=0
   	            if denom_num==num_num:
       	                for i in range(num_num):
           	            if 'f/i' in dthzlist[i]: im_denom+=1
                            if 'f/i' in nthzlist[i]: im_num+=1
                        if not im_num==0: 
                            print 'WARNING: Imaginary frequency found in the local minimum! Please check the OUTCAR of %s!'%vdir_num[freq]
                        elif im_denom==0:
                            print 'WARNING: No imaginary frequency found in the saddle point! Please check the OUTCAR of %s!'%vdir_denom[freq]
                        elif im_denom>1:
                            print 'WARNING: More than 1 imaginary frequencies found in the saddle point! Please check the OUTCAR of %s!'%vdir_denom[freq]
       	            else:
       	                print 'WARNING: Numbers of frequencies at local minimum and at saddle point are not equal! Please check the OUTCAR of %s and %s!'%(vdir_num[freq],vdir_denom[freq])
                    v_num[freq]=v_denom[freq]=1.0
                    for i in range(num_num):
                        if not 'f/i' in nthzlist[i]:
                            v_num[freq]*=float(self.getinfo(nthzlist[i])[3])
                    for i in range(denom_num):    
                        if not 'f/i' in dthzlist[i]:
                            v_denom[freq]*=float(self.getinfo(dthzlist[i])[3])
		v[freq]=v_num[freq]/v_denom[freq]
        return v


class DiffCoeff(ParsingInputFiles):   
    """Calculating diffusion coefficient.
        Attributes:
            self.init_values <dict of dict>: Initializing values needed to calculate D
            self.calculatingD <void>: printing system information and calculating D in desired frequency model.
        D~1/T data file - Diffusivity.txt is generated.
    """     

    def init_values(self):
        """Evaluating the values that are required in the D calculation, including 
        frequency type, lattice constant, energy barriers, attempt frequencies and HVf (and HB).
        """
        values = dict()      
        vdir_num = dict()
        vdir_denom = dict()
        Edir_saddle = dict()
        Edir_min = dict()
        vdir = self.get_item_name('v')        
        Edir = self.get_item_name('E')
        Hdir = self.get_item_name('H')
        latt = self.get_item_name('lattice')
        model = self.get_item_name('type')
        plotdisplay = self.get_item_name('plotdisplay')
        if len(plotdisplay) == 0: #empty dictionary
            plotdisplay=""
        values['plotdisplay'] = plotdisplay
        numatom = self.get_latt(model,latt)['No.']
        values['a'] = self.get_latt(model,latt)['a']
        values['type'] = model
        if model==8: values['c'] = self.get_latt(model,latt)['c']

        for i in vdir.keys(): 
            if not len(vdir[i])==1:
                try: vdir_num[i]=float(vdir[i][0])
                except ValueError: vdir_num[i] = vdir[i][0]
                try: vdir_denom[i]=float(vdir[i][1])
                except ValueError: vdir_denom[i] = vdir[i][1]
        for i in Edir.keys():
            if not len(Edir[i])==1:
                try: Edir_saddle[i]=float(Edir[i][1])
                except ValueError: Edir_saddle[i] = Edir[i][1]
                try: Edir_min[i] = float(Edir[i][0])
                except ValueError: Edir_min[i] = Edir[i][0]
       
        values['enebarr'] = self.get_barrier(Edir,Edir_saddle,Edir_min)
        values['v'] = self.get_v(vdir,vdir_num,vdir_denom)
	values['HVf'] = self.get_HB_and_HVf(Hdir,numatom,'HVf')
        if model==8:
            values['HB'] = self.get_HB_and_HVf(Hdir,numatom,'HB')
        return values
    
    def calculatingD(self):
        """Printing system information and calculating D accordingly.
        D~1/T relation is printed and stored in the data file of Diffusivity.txt.
        """
        kB = 11604.52
        jfreq = dict()
        values = self.init_values()
        model = values['type']
        enebarr = values['enebarr']
        v = values['v']
        a = values['a'] 
        plotdisplay = values['plotdisplay']
        HVf = values['HVf'] 
        if model==8: 
            c = values['c'] 
            HB = values['HB'] 
	# print all system information
        if model==5:
            print "FCC Five-Frequency Diffusion Model"
            print "FCC lattice constant [Angstrom]: {0:.4f}".format(a*10**8)
            print "Energy Barriers [eV]:       E0: {E0:.4f}  E1: {E1:.4f}  E2: {E2:.4f}  E3: {E3:.4f}  E4: {E4:.4f}".format(**enebarr)
            print "Attempt Frequencies [THz]:  v0: {v0:.4f}  v1: {v1:.4f}  v2: {v2:.4f}  v3: {v3:.4f}  v4: {v4:.4f}".format(**v)
            print "Vacancy Formation Energy [eV]: {0:.4f}\n".format(HVf)
            print ""
        if model==8:
            print "HCP Eight-Frequency Diffusion Model"
            print "HCP basal lattice constant  [Angstrom]: {0:.4f}".format(a*10**8)
            print "HCP c-axis lattice constant [Angstrom]: {0:.4f}".format(c*10**8)
            print "Energy Barriers [eV]:       Ea: {Ea:.4f}  Eb: {Eb:.4f}  Ec: {Ec:.4f}  EX: {EX:.4f}  E'a: {Eap:.4f}  E'b: {Ebp:.4f}  E'c: {Ecp:.4f}  E'X: {EXp:.4f}".format(**enebarr)
            print "Attempt Frequencies [THz]:  va: {va:.4f}  vb: {vb:.4f}  vc: {vc:.4f}  vX: {vX:.4f}  v'a: {vap:.4f}  v'b: {vbp:.4f}  v'c: {vcp:.4f}  v'X: {vXp:.4f}".format(**v)
            print "Vacancy Formation Energy [eV]: {0:.4f}".format(HVf)
            print "Vacancy-Solute Binding Energy [eV]:  {0:.4f}\n".format(HB)
        
        
        try:
            temp = self.get_item_name('temp')['temp']
            tempstart = temp[0]
            tempstep = temp[1]
            tempend = temp[2]
        except: tempstart = 0.0; tempstep = 0.1; tempend = 2.0 # default temperature range
        if model==5:
            fp = open('Diffusivity.txt','w+')
            fp.write('1000/T(K^(-1))    D(cm^2/s)\n')
            print '1000/T(K^(-1))    D(cm^2/s)'
            D = []
            f0 = 0.7815
            t = tempstart
            i = 0
            while t<tempend+tempstep:
                for freq in ['v0','v1','v2','v3','v4']:
                    if t==0.0: jfreq[freq] = v[freq]*10**12
                    else: 
                        T=1000./t
                        jfreq[freq] = v[freq]* 10**12 * np.exp(-enebarr[freq.replace('v','E')]*kB/T)
                alpha = jfreq['v4']/jfreq['v0']
                F_num = 10*np.power(alpha,4) + 180.5*np.power(alpha,3) + 927*np.power(alpha,2) + 1341*alpha
                F_denom = 2*np.power(alpha,4) + 40.2*np.power(alpha,3) + 254*np.power(alpha,2) + 597*alpha + 435
                FX = 1-(1.0/7.0)*(F_num/F_denom)
                f2_num = 1+3.5*FX *(jfreq['v3']/jfreq['v1'])
                f2_denom = 1+(jfreq['v2']/jfreq['v1']) + 3.5*FX*(jfreq['v3']/jfreq['v1'])
                f2 = f2_num / f2_denom       
                if t==0.0: Vacconc=1.0
                else: Vacconc = np.exp(-kB*HVf/T)       
                Dself = f0*Vacconc*a**2*jfreq['v0']
                D.append(Dself*f2*jfreq['v2']*jfreq['v4']/f0/jfreq['v0']/jfreq['v3'])
                fp.write('%f  %E\n'%(t,D[i]))
                print '%f  %E'%(t,D[i])
                t = t + tempstep    
                i = i + 1
            if plotdisplay.lower() == "none": #No plot display
                pass
            else:
                import matplotlib
                if len(plotdisplay) == 0:
                    pass
                else:
                    matplotlib.use(plotdisplay)
                import matplotlib.pyplot as plt
                tvals = np.linspace(tempstart, tempend, i)
                plt.plot(tvals, np.log10(D), '.')
                plt.xlabel('$10^3/T$ (K$^{-1}$)')
                plt.ylabel('log$_{10}$$D$ (cm$^2$/s)')
                plt.savefig('Diffusivity.png')
                plt.show()

            
        elif model==8:
            fp = open('Diffusivity.txt','w+')
            fp.write('1000/T(K^(-1))    D_basal(cm^2/s)    D_c-axis(cm^2/s)\n')
            print '1000/T(K^(-1))    D_basal(cm^2/s)    D_c-axis(cm^2/s)'
            Dperp = []
            Dpara = []
            F = 0.736
            lambda1 = a
            lambda2b = a/(3**0.5)
            t = tempstart
            i = 0
            while t<tempend+tempstep:
                for freq in ['va','vb','vc','vX','vap','vbp','vcp','vXp']:
                    if t==0.0: jfreq[freq] = v[freq]*10**12
                    else:
                        T=1000./t
                        jfreq[freq] = v[freq] * 10**12 * np.exp(-enebarr[freq.replace('v','E')]*kB/T)
                f1z = (2*jfreq['vap']+7*F*jfreq['vcp'])/(2*jfreq['vap']+7*F*jfreq['vcp']+2*jfreq['vXp'])
                S1b_num = (3**0.5)*jfreq['vX']/jfreq['va']*lambda1+jfreq['vXp']/jfreq['vap']*lambda2b*(2+jfreq['vb']/jfreq['va']+7*F*jfreq['vc']/jfreq['va']+2*jfreq['vX']/jfreq['va'])
                S1b_denom = 3 - (2+jfreq['vb']/jfreq['va']+7*F*jfreq['vc']/jfreq['va']+2*jfreq['vX']/jfreq['va'])*(2+3*jfreq['vbp']/jfreq['vap']+7*F*jfreq['vcp']/jfreq['vap']+2*jfreq['vXp']/jfreq['vap']-1./(2+3*jfreq['vb']/jfreq['va']+7*F*jfreq['vc']/jfreq['va']+jfreq['vX']/jfreq['va']))
                S1b = S1b_num / S1b_denom
                S2x=((3**0.5)*S1b-jfreq['vX']/jfreq['va']*lambda1)/(2+jfreq['vb']/jfreq['va']+7*F*jfreq['vc']/jfreq['va']+2*jfreq['vX']/jfreq['va'])
                f1b=1+2*S1b/lambda2b
                f2x=1+2*S2x/lambda1
                if t==0.0: C2=1.0
                else: C2=np.exp(-kB*(HVf+HB)/T)
                Dperp.append(a**2*(3*f2x*jfreq['vX']+f1b*jfreq['vXp'])*C2/2)
                Dpara.append(c**2*f1z*jfreq['vXp']*C2*0.75)
                fp.write('%f  %E  %E\n'%(t,Dperp[i],Dpara[i]))
                print '%f  %E  %E'%(t,Dperp[i],Dpara[i])
                t = t + tempstep
                i = i + 1
            if plotdisplay.lower() == "none":
                pass
            else:
                import matplotlib
                if len(plotdisplay) == 0:
                    pass
                else:
                    matplotlib.use(plotdisplay)
                import matplotlib.pyplot as plt
                tvals = np.linspace(tempstart, tempend, i)
                plt.plot(tvals, np.log10(Dperp), '.', tvals, np.log10(Dpara), '+')
                plt.xlabel('$10^3/T$ (K$^{-1}$)')
                plt.ylabel('log$_{10}$$D$ (cm$^2$/s)')
                plt.savefig('Diffusivity.png')
                plt.show()




import numpy

def fingerprint_dist(fingerprint1, fingerprint2):
    """Function to calculate the cosine distance between two fingerprint functions
    """
    f1=numpy.array(fingerprint1)
    f1mag=numpy.linalg.norm(f1)
    f2=numpy.array(fingerprint2)
    f2mag=numpy.linalg.norm(f2)
    dist=0.5*(1-numpy.dot(f1,f2)/(f1mag*f2mag))
    return dist

